@@ -1,15 +1,15 @@
-/********************************************************************************/
-/* Copyright (c) 2004                                                           */
-/* Daniel Sleator, David Temperley, and John Lafferty                           */
-/* All rights reserved                                                          */
-/*                                                                              */
-/* Use of the link grammar parsing system is subject to the terms of the        */
-/* license set forth in the LICENSE file included with this software,           */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html           */
-/* This license allows free redistribution and use in source and binary         */
-/* forms, with or without modification, subject to certain conditions.          */
-/*                                                                              */
-/********************************************************************************/
+/*************************************************************************/
+/* Copyright (c) 2004                                                    */
+/* Daniel Sleator, David Temperley, and John Lafferty                    */
+/* All rights reserved                                                   */
+/*                                                                       */
+/* Use of the link grammar parsing system is subject to the terms of the */
+/* license set forth in the LICENSE file included with this software,    */
+/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* This license allows free redistribution and use in source and binary  */
+/* forms, with or without modification, subject to certain conditions.   */
+/*                                                                       */
+/*************************************************************************/
 
 
 #include <stdarg.h>
@@ -86,34 +86,43 @@ struct Links_to_patch_struct {
 	int new;	/* the new value of the end to be patched */
 };
 
-
-static Sublinkage * x_create_sublinkage(Parse_info pi) {
+void zero_sublinkage(Sublinkage *s)
+{
 	int i;
+	s->pp_info = NULL;
+	s->violation = NULL;
+	for (i=0; i<s->num_links; i++) s->link[i] = NULL;
+
+	memset(&s->pp_data, 0, sizeof(PP_data));
+}
+
+static Sublinkage * x_create_sublinkage(Parse_info pi)
+{
 	Sublinkage *s = (Sublinkage *) xalloc (sizeof(Sublinkage));
 	s->link = (Link *) xalloc(MAX_LINKS*sizeof(Link));
-	s->pp_info = NULL;
-	s->violation = NULL;
-	for (i=0; i<MAX_LINKS; i++)		s->link[i] = NULL;
+	s->num_links = MAX_LINKS;
+
+	zero_sublinkage(s);
+
 	s->num_links = pi->N_links;
 	assert(pi->N_links < MAX_LINKS, "Too many links");
 	return s;
 }
 
-
-static Sublinkage * ex_create_sublinkage(Parse_info pi) {
-	int i;
+static Sublinkage * ex_create_sublinkage(Parse_info pi)
+{
 	Sublinkage *s = (Sublinkage *) exalloc (sizeof(Sublinkage));
 	s->link = (Link *) exalloc(pi->N_links*sizeof(Link));
-	s->pp_info = NULL;
-	s->violation = NULL;
-	for (i=0; i<pi->N_links; i++) s->link[i] = NULL;
 	s->num_links = pi->N_links;
+
+	zero_sublinkage(s);
+
 	assert(pi->N_links < MAX_LINKS, "Too many links");
 	return s;
 }
 
-
-static void free_sublinkage(Sublinkage *s) {
+static void free_sublinkage(Sublinkage *s)
+{
 	int i;
 	for (i=0; i<MAX_LINKS; i++) {
 		if (s->link[i]!=NULL) exfree_link(s->link[i]);
@@ -122,13 +131,17 @@ static void free_sublinkage(Sublinkage *s) {
 	xfree(s, sizeof(Sublinkage));
 }
 
-static void replace_link_name(Link l, char *s) {
-	exfree(l->name, sizeof(char)*(strlen(l->name)+1));
-	l->name = (char *) exalloc(sizeof(char)*(strlen(s)+1));
-	strcpy(l->name, s);
+static void replace_link_name(Link l, const char *s)
+{
+	char * t;
+	exfree((char *) l->name, sizeof(char)*(strlen(l->name)+1));
+	t = (char *) exalloc(sizeof(char)*(strlen(s)+1));
+	strcpy(t, s);
+	l->name = t;
 }
 
-static void copy_full_link(Link *dest, Link src) {
+static void copy_full_link(Link *dest, Link src)
+{
 	if (*dest != NULL) exfree_link(*dest);
 	*dest = excopy_link(src);
 }
@@ -611,13 +624,14 @@ static int disjunct_cost(Parse_info pi) {
 	return lcost;
 }
 
-static int strictly_smaller_name(char * s, char * t) {
-/*
-   Returns TRUE if string s represents a strictly smaller match set
-   than does t.  An almost identical function appears in and.c.
-   The difference is that here we don't require s and t to be the
-   same length.
-*/
+/**
+ * Returns TRUE if string s represents a strictly smaller match set
+ * than does t.  An almost identical function appears in and.c.
+ * The difference is that here we don't require s and t to be the
+ * same length.
+ */
+static int strictly_smaller_name(const char * s, const char * t)
+{
 	int strictness, ss, tt;
 	strictness = 0;
 	while ((*s!='\0') || (*t!='\0')) {
@@ -643,11 +657,12 @@ static int strictly_smaller_name(char * s, char * t) {
 	return (strictness > 0);
 }
 
-static void compute_link_names(Sentence sent) {
-/*
-   The name of the link is set to be the GCD of the names of
-   its two endpoints.
-*/
+/**
+ * The name of the link is set to be the GCD of the names of
+ * its two endpoints.
+ */
+static void compute_link_names(Sentence sent)
+{
 	int i;
 	Parse_info pi = sent->parse_info;
 
@@ -657,20 +672,20 @@ static void compute_link_names(Sentence sent) {
 	}
 }
 
+/**
+ * This fills in the sublinkage->link[].name field.  We assume that
+ * link_array[].name have already been filled in.  As above, in the
+ * standard case, the name is just the GCD of the two end points.
+ * If pluralization has occurred, then we want to use the name
+ * already in link_array[].name.  We detect this in two ways.
+ * If the endpoints don't match, then we know pluralization
+ * has occured.  If they do, but the name in link_array[].name
+ * is *less* restrictive, then pluralization must have occured.
+ */
 static void compute_pp_link_names(Sentence sent, Sublinkage *sublinkage)
 {
-/*
-   This fills in the sublinkage->link[].name field.  We assume that
-   link_array[].name have already been filled in.  As above, in the
-   standard case, the name is just the GCD of the two end points.
-   If pluralization has occurred, then we want to use the name
-   already in link_array[].name.  We detect this in two ways.
-   If the endpoints don't match, then we know pluralization
-   has occured.  If they do, but the name in link_array[].name
-   is *less* restrictive, then pluralization must have occured.
-*/
 	int i;
-	char * s;
+	const char * s;
 	Parse_info pi = sent->parse_info;
 
 	for (i=0; i<pi->N_links; i++)
@@ -902,11 +917,13 @@ void extract_thin_linkage(Sentence sent, Parse_Options opts, Linkage linkage)
 }
 
 
-void extract_fat_linkage(Sentence sent, Parse_Options opts, Linkage linkage) {
-  /* This procedure mimics analyze_fat_linkage in order to
-	 extract the sublinkages and copy them to the Linkage
-	 data structure passed in.
-	 */
+/**
+ * This procedure mimics analyze_fat_linkage in order to
+ * extract the sublinkages and copy them to the Linkage
+ * data structure passed in.
+ */
+void extract_fat_linkage(Sentence sent, Parse_Options opts, Linkage linkage)
+{
 	int i, j, N_thin_links;
 	DIS_node *d_root;
 	int num_sublinkages;
@@ -949,6 +966,7 @@ void extract_fat_linkage(Sentence sent, Parse_Options opts, Linkage linkage) {
 	linkage->sublinkage =
 		(Sublinkage *) exalloc(sizeof(Sublinkage)*num_sublinkages);
 	for (i=0; i<num_sublinkages; ++i) {
+		linkage->sublinkage[i].link = NULL;
 		linkage->sublinkage[i].pp_info = NULL;
 		linkage->sublinkage[i].violation = NULL;
 	}
