@@ -1,15 +1,15 @@
-/********************************************************************************/
-/* Copyright (c) 2004                                                           */
-/* Daniel Sleator, David Temperley, and John Lafferty                           */
-/* All rights reserved                                                          */
-/*                                                                              */
-/* Use of the link grammar parsing system is subject to the terms of the        */
-/* license set forth in the LICENSE file included with this software,           */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html           */
-/* This license allows free redistribution and use in source and binary         */
-/* forms, with or without modification, subject to certain conditions.          */
-/*                                                                              */
-/********************************************************************************/
+/*************************************************************************/
+/* Copyright (c) 2004                                                    */
+/* Daniel Sleator, David Temperley, and John Lafferty                    */
+/* All rights reserved                                                   */
+/*                                                                       */
+/* Use of the link grammar parsing system is subject to the terms of the */
+/* license set forth in the LICENSE file included with this software,    */
+/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* This license allows free redistribution and use in source and binary  */
+/* forms, with or without modification, subject to certain conditions.   */
+/*                                                                       */
+/*************************************************************************/
 
 #include <link-grammar/api.h>
 
@@ -351,25 +351,23 @@ void initialize_conjunction_tables(Sentence sent) {
 	}
 }
 
-static int and_connector_hash(Connector * c, int i) {
-/* This hash function that takes a connector and a seed value i.
-   It only looks at the leading upper case letters of
-   the string, and the label.  This ensures that if two connectors
-   match, then they must hash to the same place.
-*/
-	const char * s;
-	s = c->string;
-
-	i = i + (i<<1) + randtable[(c->label + i) & (RTSIZE-1)];
-	while(isupper((int)*s)) {
-		i = i + (i<<1) + randtable[(*s + i) & (RTSIZE-1)];
-		s++;
-	}
+/**
+ * This hash function that takes a connector and a seed value i.
+ * It only looks at the leading upper case letters of
+ * the string, and the label.  This ensures that if two connectors
+ * match, then they must hash to the same place.
+ */
+static int and_connector_hash(Connector * c, int i)
+{
+	i = connector_hash(c, i);
 	return (i & (HT_SIZE-1));
 }
 
-static int and_hash_disjunct(Disjunct *d) {
-/* This is a hash function for disjuncts */
+/**
+ * This is a hash function for disjuncts
+ */
+static int and_hash_disjunct(Disjunct *d)
+{
 	int i;
 	Connector *e;
 	i = 0;
@@ -402,32 +400,27 @@ static int is_appropriate(Sentence sent, Disjunct * d) {
 	return TRUE;
 }
 
+/**
+ * Two connectors are said to be of the same type if they have
+ * the same label, and the initial upper case letters of their
+ * strings match.
+ */
 static int connector_types_equal(Connector * c1, Connector * c2)
 {
-/* Two connectors are said to be of the same type if they have
-   the same label, and the initial upper case letters of their
-   strings match.
-*/
-	const char * s, * t;
 	if (c1->label != c2->label) return FALSE;
-	s = c1->string;
-	t = c2->string;
-	while(isupper((int)*s) || isupper((int)*t)) {
-		if (*s != *t) return FALSE;
-		s++;
-		t++;
-	}
-	return TRUE;
+	return utf8_upper_match(c1->string, c2->string);
 }
 
-static int disjunct_types_equal(Disjunct * d1, Disjunct * d2) {
-/* Two disjuncts are said to be the same type if they're the same
-   ignoring the multi fields, the priority fields, and the subscripts
-   of the connectors (and the string field of the disjunct of course).
-   Disjuncts of the same type are located in the same label_table list.
-
-   This returns TRUE if they are of the same type.
-*/
+/**
+ * Two disjuncts are said to be the same type if they're the same
+ * ignoring the multi fields, the priority fields, and the subscripts
+ * of the connectors (and the string field of the disjunct of course).
+ * Disjuncts of the same type are located in the same label_table list.
+ *
+ * This returns TRUE if they are of the same type.
+ */
+static int disjunct_types_equal(Disjunct * d1, Disjunct * d2)
+{
 	Connector *e1, *e2;
 
 	e1 = d1->left;
@@ -699,8 +692,9 @@ static void extract_all_fat_links(Sentence sent, Disjunct * d) {
 static char * stick_in_one_connector(char *s, Connector *c, int len)
 {
 	const char * t;
-	for (t = c->string; isupper((int)*t); t++)
-	  ;
+
+	t = skip_utf8_upper(c->string);
+
 	while (*t != '\0') {
 		*s++ = *t++;
 		len--;
@@ -714,15 +708,16 @@ static char * stick_in_one_connector(char *s, Connector *c, int len)
 	return s;
 }
 
-static void compute_matchers_for_a_label(Sentence sent, int k) {
-/* This takes a label k, modifies the list of disjuncts with that
-   label.  For each such disjunct, it computes the string that
-   will be used in the fat connector that represents it.
-
-   The only hard part is finding the length of each of the strings
-   so that "*" can be put in.  A better explanation will have to wait.
-*/
-
+/**
+ * This takes a label k, modifies the list of disjuncts with that
+ * label.  For each such disjunct, it computes the string that
+ * will be used in the fat connector that represents it.
+ *
+ * The only hard part is finding the length of each of the strings
+ * so that "*" can be put in.  A better explanation will have to wait.
+ */
+static void compute_matchers_for_a_label(Sentence sent, int k)
+{
 	int * lengths;
 	int N_connectors, i, j, tot_len;
 	Connector * c;
@@ -741,15 +736,14 @@ static void compute_matchers_for_a_label(Sentence sent, int k) {
 	while(d != NULL) {
 		i = 0;
 		for (c=d->left; c != NULL; c = c->next) {
-			cs = c->string;
-			while(isupper((int)*cs)) cs++;
+         cs = skip_utf8_upper(c->string);
 			j = strlen(cs);
 			if (j > lengths[i]) lengths[i] = j;
 			i++;
 		}
 		for (c=d->right; c != NULL; c = c->next) {
 			cs = c->string;
-			while(isupper((int)*cs)) cs++;
+			cs = skip_utf8_upper(cs);
 			j = strlen(cs);
 			if (j > lengths[i]) lengths[i] = j;
 			i++;
@@ -811,11 +805,12 @@ void print_AND_statistics(Sentence sent) {
 	printf("Number of equality tests: %d\n", STAT_calls_to_equality_test);
 }
 
+/**
+ * Fill in the fields of c for the disjunct.  This must be in
+ * the table data structures.  The label field and the string field
+ * are filled in appropriately.  Priority is set to UP_priority.
+ */
 static void connector_for_disjunct(Sentence  sent, Disjunct * d, Connector * c) {
-/* Fill in the fields of c for the disjunct.  This must be in
-   the table data structures.  The label field and the string field
-   are filled in appropriately.  Priority is set to UP_priority.
-*/
 	int h;
 	Disjunct * d1=NULL;
 	Label_node * lp;
@@ -826,21 +821,14 @@ static void connector_for_disjunct(Sentence  sent, Disjunct * d, Connector * c)
 		d1 = sent->and_data.label_table[lp->label];
 		if (disjunct_types_equal(d,d1)) break;
 	}
-	assert(lp!=NULL, "A disjunct I inserted was not there. (1)");
-/*
-  I don't know what these lines were for.  I replaced them by
-  the above assertion.
-	if (lp == NULL) {
-		printf("error: A disjunct I inserted was not there\n");
-		lp = lp->next;  (to force an error)
-	}
-*/
+	assert(lp != NULL, "A disjunct I inserted was not there. (1)");
+
 	while(d1 != NULL) {
 		if (disjuncts_equal_AND(d1, d)) break;
 		d1 = d1->next;
 	}
 
-	assert(d1!=NULL, "A disjunct I inserted was not there. (2)");
+	assert(d1 != NULL, "A disjunct I inserted was not there. (2)");
 
 	c->label = lp->label;
 	c->string = d1->string;
@@ -849,13 +837,15 @@ static void connector_for_disjunct(Sentence  sent, Disjunct * d, Connector * c)
 }
 
 
-static Disjunct * build_fat_link_substitutions(Sentence sent, Disjunct *d) {
-/* This function allocates and returns a list of disjuncts.
-   This is the one obtained by substituting each contiguous
-   non-empty subrange of d (incident on the center) by an appropriate
-   fat link, in two possible positions.  Does not effect d.
-   The cost of d is inherited by all of the disjuncts in the result.
-*/
+/**
+ * This function allocates and returns a list of disjuncts.
+ * This is the one obtained by substituting each contiguous
+ * non-empty subrange of d (incident on the center) by an appropriate
+ * fat link, in two possible positions.  Does not effect d.
+ * The cost of d is inherited by all of the disjuncts in the result.
+ */
+static Disjunct * build_fat_link_substitutions(Sentence sent, Disjunct *d)
+{
 	Connector * cl, * cr, *tl, *tr, *wc, work_connector;
 	Disjunct *d1, *wd, work_disjunct, *d_list;
 	if (d==NULL) return NULL;
