@@ -12,6 +12,7 @@
 /*************************************************************************/
 
 #include <link-grammar/api.h>
+#include "preparation.h"
 
 /***************************************************************
 *
@@ -19,11 +20,8 @@
 *
 ****************************************************************/
 
-void		  free_sentence_disjuncts(Sentence sent);
-void		  free_post_processing(Sentence sent);
-
-
-static int VDAL_compare_parse(Linkage_info * p1, Linkage_info * p2) {
+static int VDAL_compare_parse(Linkage_info * p1, Linkage_info * p2)
+{
 	/* for sorting the linkages in postprocessing */
 	if (p1->N_violations != p2->N_violations) {
 		return (p1->N_violations - p2->N_violations);
@@ -91,7 +89,8 @@ int parse_options_delete(Parse_Options  opts) {
 	return 0;
 }
 
-void parse_options_set_cost_model_type(Parse_Options opts, int cm) {
+void parse_options_set_cost_model_type(Parse_Options opts, int cm)
+{
 	switch(cm) {
 	case VDAL:
 		opts->cost_model.type = VDAL;
@@ -102,7 +101,8 @@ void parse_options_set_cost_model_type(Parse_Options opts, int cm) {
 	}
 }
 
-void parse_options_set_verbosity(Parse_Options opts, int dummy) {
+void parse_options_set_verbosity(Parse_Options opts, int dummy)
+{
 	opts->verbosity = dummy;
 	verbosity = opts->verbosity;
 	/* this is one of the only global variables. */
@@ -413,7 +413,7 @@ failure:
 	return NULL;
 }
 
-Dictionary dictionary_create_lang(char * lang)
+Dictionary dictionary_create_lang(const char * lang)
 {
 	Dictionary dictionary;
 
@@ -544,21 +544,6 @@ Sentence sentence_create(char *input_string, Dictionary dict)
 	return sent;
 }
 
-void sentence_delete(Sentence sent) 
-{
-	/* free_andlists(sent); */
-	free_sentence_disjuncts(sent);
-	free_sentence_expressions(sent);
-	string_set_delete(sent->string_set);
-	free_parse_set(sent);
-	free_post_processing(sent);
-	post_process_close_sentence(sent->dict->postprocessor);
-	free_deletable(sent);
-	free_effective_dist(sent);
-	xfree(sent->is_conjunction, sizeof(char)*sent->length);
-	xfree((char *) sent, sizeof(struct Sentence_s));
-}
-
 static void free_andlists(Sentence sent) 
 {
 	int L;
@@ -577,6 +562,31 @@ static void free_andlists(Sentence sent)
 	/* printf("\n"); */
 }
 
+static void free_post_processing(Sentence sent)
+{
+	if (sent->link_info != NULL) {
+		/* postprocessing must have been done */
+		free_andlists(sent);
+		xfree((char *) sent->link_info, sent->num_linkages_alloced*sizeof(Linkage_info));
+		sent->link_info = NULL;
+	}
+}
+
+void sentence_delete(Sentence sent) 
+{
+	/* free_andlists(sent); */
+	free_sentence_disjuncts(sent);
+	free_sentence_expressions(sent);
+	string_set_delete(sent->string_set);
+	free_parse_set(sent);
+	free_post_processing(sent);
+	post_process_close_sentence(sent->dict->postprocessor);
+	free_deletable(sent);
+	free_effective_dist(sent);
+	xfree(sent->is_conjunction, sizeof(char)*sent->length);
+	xfree((char *) sent, sizeof(struct Sentence_s));
+}
+
 int sentence_length(Sentence sent)
 {
 	if (!sent) return 0;
@@ -626,8 +636,8 @@ int sentence_nth_word_has_disjunction(Sentence sent, int i) {
 ****************************************************************/
 
 
-static void post_process_linkages(Sentence sent, Parse_Options opts) {
-
+static void post_process_linkages(Sentence sent, Parse_Options opts)
+{
 	int *indices;
 	int in, block_bottom, block_top;
 	int N_linkages_found, N_linkages_alloced;
@@ -662,14 +672,14 @@ static void post_process_linkages(Sentence sent, Parse_Options opts) {
 	}
 
 	if (N_linkages_found > opts->linkage_limit)
-	  {
+	{
 		N_linkages_alloced = opts->linkage_limit;
 		if (opts->verbosity > 1) {
 		  fprintf(stdout,
 				  "Warning: Considering a random subset of %d of %d linkages\n",
 				  N_linkages_alloced, N_linkages_found);
 		}
-	  }
+	}
 	else N_linkages_alloced = N_linkages_found;
 
 	link_info=(Linkage_info *)xalloc(N_linkages_alloced * sizeof(Linkage_info));
@@ -768,17 +778,10 @@ static void post_process_linkages(Sentence sent, Parse_Options opts) {
 	/*if(N_valid_linkages == 0) free_andlists(sent); */
 }
 
-void free_post_processing(Sentence sent) {
-	if (sent->link_info != NULL) {
-		/* postprocessing must have been done */
-		free_andlists(sent);
-		xfree((char *) sent->link_info, sent->num_linkages_alloced*sizeof(Linkage_info));
-		sent->link_info = NULL;
-	}
-}
-
-int sentence_parse(Sentence sent, Parse_Options opts) {
+int sentence_parse(Sentence sent, Parse_Options opts)
+{
 	int nl;
+	s64 total;
 
 	verbosity = opts->verbosity;
 
@@ -798,14 +801,21 @@ int sentence_parse(Sentence sent, Parse_Options opts) {
 	init_table(sent);
 
 	/* A parse set may have been already been built for this sentence,
-	   if it was previously parsed.  If so we free it up before building another.  */
+	 * if it was previously parsed.  If so we free it up before
+	 * building another.  */
 	free_parse_set(sent);
 	init_x_table(sent);
 
-	for (nl = opts->min_null_count;
-		 (nl<=opts->max_null_count) && (!resources_exhausted(opts->resources)); ++nl) {
+	for (nl = opts->min_null_count; nl<=opts->max_null_count ; ++nl)
+	{
+		if (resources_exhausted(opts->resources)) break;
 		sent->null_count = nl;
-		sent->num_linkages_found = parse(sent, sent->null_count, opts);
+		total = parse(sent, sent->null_count, opts);
+
+		/* Give up if the parse count is overflowing */
+		if (PARSE_NUM_OVERFLOW < total) break;
+
+		sent->num_linkages_found = (int) total;
 		print_time(opts, "Counted parses");
 		post_process_linkages(sent, opts);
 		if (sent->num_valid_linkages > 0) break;
@@ -824,16 +834,17 @@ int sentence_parse(Sentence sent, Parse_Options opts) {
 *
 ****************************************************************/
 
-Linkage linkage_create(int k, Sentence sent, Parse_Options opts) {
+Linkage linkage_create(int k, Sentence sent, Parse_Options opts)
+{
 	Linkage linkage;
 
-	assert((k < sent->num_linkages_post_processed) && (k >= 0), "index out of range");
+	if ((k >= sent->num_linkages_post_processed) || (k < 0)) return NULL;
 
 	/* Using exalloc since this is external to the parser itself. */
 	linkage = (Linkage) exalloc(sizeof(struct Linkage_s));
 
 	linkage->num_words = sent->length;
-	linkage->word = (char **) exalloc(linkage->num_words*sizeof(char *));
+	linkage->word = (const char **) exalloc(linkage->num_words*sizeof(char *));
 	linkage->current = 0;
 	linkage->num_sublinkages=0;
 	linkage->sublinkage = NULL;
@@ -870,38 +881,47 @@ int linkage_set_current_sublinkage(Linkage linkage, int index) {
 	return 1;
 }
 
-static void exfree_pp_info(PP_info ppi) {
+static void exfree_pp_info(PP_info *ppi)
+{
 	int i;
-	for (i=0; i<ppi.num_domains; ++i) {
-		exfree(ppi.domain_name[i], strlen(ppi.domain_name[i])+1);
+	for (i=0; i<ppi->num_domains; ++i) {
+		exfree(ppi->domain_name[i], strlen(ppi->domain_name[i])+1);
 	}
-	if (ppi.num_domains > 0)
-		exfree(ppi.domain_name, sizeof(char *)*ppi.num_domains);
+	if (ppi->num_domains > 0)
+		exfree(ppi->domain_name, sizeof(char *)*ppi->num_domains);
+	ppi->domain_name = NULL;
+	ppi->num_domains = 0;
 }
 
-void linkage_delete(Linkage linkage) {
+void linkage_delete(Linkage linkage)
+{
 	int i, j;
 	Sublinkage *s;
 
+	/* Can happen on panic timeout or user error */
+	if (NULL == linkage) return;
+
 	for (i=0; i<linkage->num_words; ++i) {
-		exfree(linkage->word[i], strlen(linkage->word[i])+1);
+		exfree((char *) linkage->word[i], strlen(linkage->word[i])+1);
 	}
 	exfree(linkage->word, sizeof(char *)*linkage->num_words);
-	for (i=0; i<linkage->num_sublinkages; ++i) {
+	for (i=0; i<linkage->num_sublinkages; ++i)
+	{
 		s = &(linkage->sublinkage[i]);
 		for (j=0; j<s->num_links; ++j) {
-		  exfree_link(s->link[j]);
+			exfree_link(s->link[j]);
 		}
 		exfree(s->link, sizeof(Link)*s->num_links);
 		if (s->pp_info != NULL) {
 			for (j=0; j<s->num_links; ++j) {
-				exfree_pp_info(s->pp_info[j]);
+				exfree_pp_info(&s->pp_info[j]);
 			}
 			exfree(s->pp_info, sizeof(PP_info)*s->num_links);
+			s->pp_info = NULL;
 			post_process_free_data(&s->pp_data);
 		}
 		if (s->violation != NULL) {
-			exfree(s->violation, sizeof(char)*(strlen(s->violation)+1));
+			exfree((char *) s->violation, sizeof(char)*(strlen(s->violation)+1));
 		}
 	}
 	exfree(linkage->sublinkage, sizeof(Sublinkage)*linkage->num_sublinkages);
@@ -937,11 +957,12 @@ static PP_info excopy_pp_info(PP_info ppi) {
 }
 
 
-static Sublinkage unionize_linkage(Linkage linkage) {
+static Sublinkage unionize_linkage(Linkage linkage)
+{
 	int i, j, num_in_union=0;
 	Sublinkage u;
 	Link link;
-	char *p;
+	const char *p;
 
 	for (i=0; i<linkage->num_sublinkages; ++i) {
 		for (j=0; j<linkage->sublinkage[i].num_links; ++j) {
@@ -950,10 +971,13 @@ static Sublinkage unionize_linkage(Linkage linkage) {
 		}
 	}
 
-	u.num_links = num_in_union;
 	u.link = (Link *) exalloc(sizeof(Link)*num_in_union);
+	u.num_links = num_in_union;
+	zero_sublinkage(&u);
+
 	u.pp_info = (PP_info *) exalloc(sizeof(PP_info)*num_in_union);
 	u.violation = NULL;
+	u.num_links = num_in_union;
 
 	num_in_union = 0;
 
@@ -965,8 +989,9 @@ static Sublinkage unionize_linkage(Linkage linkage) {
 				u.pp_info[num_in_union] = excopy_pp_info(linkage->sublinkage[i].pp_info[j]);
 				if (((p=linkage->sublinkage[i].violation) != NULL) &&
 					(u.violation == NULL)) {
-					u.violation = (char *) exalloc((strlen(p)+1)*sizeof(char));
-					strcpy(u.violation, p);
+					char *s = (char *) exalloc((strlen(p)+1)*sizeof(char));
+					strcpy(s, p);
+					u.violation = s;
 				}
 				num_in_union++;
 			}
@@ -976,9 +1001,10 @@ static Sublinkage unionize_linkage(Linkage linkage) {
 	return u;
 }
 
-int linkage_compute_union(Linkage linkage) {
+int linkage_compute_union(Linkage linkage)
+{
 	int i, num_subs=linkage->num_sublinkages;
-	Sublinkage * new_sublinkage;
+	Sublinkage * new_sublinkage, *s;
 
 	if (linkage->unionized) {
 		linkage->current = linkage->num_sublinkages-1;
@@ -997,15 +1023,13 @@ int linkage_compute_union(Linkage linkage) {
 	}
 	exfree(linkage->sublinkage, sizeof(Sublinkage)*num_subs);
 	linkage->sublinkage = new_sublinkage;
-	linkage->sublinkage[num_subs] = unionize_linkage(linkage);
 
-	/* The domain data will not be needed for the union -- zero it out */
-	linkage->sublinkage[num_subs].pp_data.N_domains=0;
-	linkage->sublinkage[num_subs].pp_data.length=0;
-	linkage->sublinkage[num_subs].pp_data.links_to_ignore=NULL;
-	for (i=0; i<MAX_SENTENCE; ++i) {
-	  linkage->sublinkage[num_subs].pp_data.word_links[i] = NULL;
-	}
+	/* Zero out the new sublinkage, then unionize it. */
+	s = &new_sublinkage[num_subs];
+	s->link = NULL;
+	s->num_links = 0;
+	zero_sublinkage(s);
+	linkage->sublinkage[num_subs] = unionize_linkage(linkage);
 
 	linkage->num_sublinkages++;
 
@@ -1077,28 +1101,30 @@ int linkage_get_link_rword(Linkage linkage, int index) {
 	return link->r;
 }
 
-char * linkage_get_link_label(Linkage linkage, int index) {
+const char * linkage_get_link_label(Linkage linkage, int index) {
 	Link link;
 	if (!verify_link_index(linkage, index)) return NULL;
 	link = linkage->sublinkage[linkage->current].link[index];
 	return link->name;
 }
 
-char * linkage_get_link_llabel(Linkage linkage, int index) {
+const char * linkage_get_link_llabel(Linkage linkage, int index)
+{
 	Link link;
 	if (!verify_link_index(linkage, index)) return NULL;
 	link = linkage->sublinkage[linkage->current].link[index];
 	return link->lc->string;
 }
 
-char * linkage_get_link_rlabel(Linkage linkage, int index) {
+const char * linkage_get_link_rlabel(Linkage linkage, int index)
+{
 	Link link;
 	if (!verify_link_index(linkage, index)) return NULL;
 	link = linkage->sublinkage[linkage->current].link[index];
 	return link->rc->string;
 }
 
-char ** linkage_get_words(Linkage linkage) {
+const char ** linkage_get_words(Linkage linkage) {
 	return linkage->word;
 }
 
@@ -1106,7 +1132,7 @@ Sentence linkage_get_sentence(Linkage linkage) {
 	return linkage->sent;
 }
 
-char * linkage_get_word(Linkage linkage, int w) {
+const char * linkage_get_word(Linkage linkage, int w) {
 	return linkage->word[w];
 }
 
@@ -1140,7 +1166,7 @@ char ** linkage_get_link_domain_names(Linkage linkage, int index) {
 	return pp_info.domain_name;
 }
 
-char * linkage_get_violation_name(Linkage linkage) {
+const char * linkage_get_violation_name(Linkage linkage) {
 	return linkage->sublinkage[linkage->current].violation;
 }
 
@@ -1156,7 +1182,8 @@ int linkage_has_inconsistent_domains(Linkage linkage) {
 	return linkage->info.inconsistent_domains;
 }
 
-void linkage_post_process(Linkage linkage, Postprocessor * postprocessor) {
+void linkage_post_process(Linkage linkage, Postprocessor * postprocessor)
+{
 	int N_sublinkages = linkage_get_num_sublinkages(linkage);
 	Parse_Options opts = linkage->opts;
 	Sentence sent = linkage->sent;
@@ -1170,7 +1197,7 @@ void linkage_post_process(Linkage linkage, Postprocessor * postprocessor) {
 		subl = &linkage->sublinkage[i];
 		if (subl->pp_info != NULL) {
 			for (j=0; j<subl->num_links; ++j) {
-				exfree_pp_info(subl->pp_info[j]);
+				exfree_pp_info(&subl->pp_info[j]);
 			}
 			post_process_free_data(&subl->pp_data);
 			exfree(subl->pp_info, sizeof(PP_info)*subl->num_links);
@@ -1181,7 +1208,7 @@ void linkage_post_process(Linkage linkage, Postprocessor * postprocessor) {
 			subl->pp_info[j].domain_name = NULL;
 		}
 		if (subl->violation != NULL) {
-			exfree(subl->violation, sizeof(char)*(strlen(subl->violation)+1));
+			exfree((char *)subl->violation, sizeof(char)*(strlen(subl->violation)+1));
 			subl->violation = NULL;
 		}
 
@@ -1216,9 +1243,9 @@ void linkage_post_process(Linkage linkage, Postprocessor * postprocessor) {
 			}
 			subl->pp_data = postprocessor->pp_data;
 			if (pp->violation != NULL) {
-				subl->violation =
-					(char *) exalloc(sizeof(char)*(strlen(pp->violation)+1));
-				strcpy(subl->violation, pp->violation);
+				char * s = (char *) exalloc(sizeof(char)*(strlen(pp->violation)+1));
+				strcpy(s, pp->violation);
+				subl->violation = s;
 			}
 		}
 	}
