@@ -1,15 +1,15 @@
-/********************************************************************************/
-/* Copyright (c) 2004                                                           */
-/* Daniel Sleator, David Temperley, and John Lafferty                           */
-/* All rights reserved                                                          */
-/*                                                                              */
-/* Use of the link grammar parsing system is subject to the terms of the        */
-/* license set forth in the LICENSE file included with this software,           */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html           */
-/* This license allows free redistribution and use in source and binary         */
-/* forms, with or without modification, subject to certain conditions.          */
-/*                                                                              */
-/********************************************************************************/
+/*************************************************************************/
+/* Copyright (c) 2004                                                    */
+/* Daniel Sleator, David Temperley, and John Lafferty                    */
+/* All rights reserved                                                   */
+/*                                                                       */
+/* Use of the link grammar parsing system is subject to the terms of the */
+/* license set forth in the LICENSE file included with this software,    */
+/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* This license allows free redistribution and use in source and binary  */
+/* forms, with or without modification, subject to certain conditions.   */
+/*                                                                       */
+/*************************************************************************/
 
 #include <stdarg.h>
 #include <link-grammar/api.h>
@@ -27,9 +27,9 @@ static String_set * phrase_ss;
 struct {
   int left;
   int right;
-  char * type;
+  const char * type;
   char domain_type;
-  char * start_link;
+  const char * start_link;
   int start_num;
   int subl;
   int canon;
@@ -60,19 +60,15 @@ static int  find_next_element(Linkage linkage,
 							  int num_elements,
 							  int num_lists);
 
-/*********************************************************/
-/*		These functions do the bulk of the actual	  */
-/* constituent-generating; they're called once for each  */
-/*					  sublinkage					   */
-/*********************************************************/
+/******************************************************
+ *        These functions do the bulk of the actual
+ * constituent-generating; they're called once for each
+ *                      sublinkage
+ *********************************************************/
 
-static int uppercompare(const char * s, const char * t) {
-  while(isupper((int) *s) || isupper((int) *t)) {
-	  if (*s != *t) return 1;
-	  s++;
-	  t++;
-  }
-  return 0;
+static inline int uppercompare(const char * s, const char * t)
+{
+	return (FALSE == utf8_upper_match(s,t));
 }
 
 static int gen_comp(Linkage linkage, int numcon_total, int numcon_subl,
@@ -648,7 +644,7 @@ static void generate_misc_word_info(Linkage linkage) {
 	   (This function is called once for each sublinkage.) */
 	
 	int l1, l2, w1, w2;
-	char * label1, * label2;
+	const char * label1, * label2;
 
 	for (w1=0; w1<linkage->num_words; w1++)
 		wordtype[w1]=NONE;
@@ -1251,7 +1247,7 @@ static char * exprint_constituent_structure(Linkage linkage, int numcon_total) {
 			/* Now, if the first character of the word was
 			   originally uppercase, we put it back that way */
 			if (sent->word[w].firstupper ==1 )
-				s[0]=toupper(s[0]);
+				upcase_utf8_str(s, s, MAX_WORD);
 			append_string(cs, "%s ", s);
 		}
 
