@@ -33,20 +33,22 @@ void count_unset_effective_distance(Sentence sent) {
 	effective_dist = NULL;
 }
 
-int match(Connector *a, Connector *b, int aw, int bw) {
-  /*
-	 Returns TRUE if s and t match according to the connector matching
-	 rules.  The connector strings must be properly formed, starting with
-	 zero or more upper case letters, followed by some other letters, and
-	 The algorithm is symmetric with respect to a and b.
-	
-	 It works as follows:  The labels must match.  The priorities must be
-	 compatible (both THIN_priority, or one UP_priority and one DOWN_priority).
-	 The sequence of upper case letters must match exactly.  After these comes
-	 a sequence of lower case letters "*"s or "^"s.  The matching algorithm
-	 is different depending on which of the two priority cases is being
-	 considered.  See the comments below.  */
-	char *s, *t;
+/*
+ * Returns TRUE if s and t match according to the connector matching
+ * rules.  The connector strings must be properly formed, starting with
+ * zero or more upper case letters, followed by some other letters, and
+ * The algorithm is symmetric with respect to a and b.
+ *
+ * It works as follows:  The labels must match.  The priorities must be
+ * compatible (both THIN_priority, or one UP_priority and one DOWN_priority).
+ * The sequence of upper case letters must match exactly.  After these comes
+ * a sequence of lower case letters "*"s or "^"s.  The matching algorithm
+ * is different depending on which of the two priority cases is being
+ * considered.  See the comments below. 
+ */
+int match(Connector *a, Connector *b, int aw, int bw)
+{
+	const char *s, *t;
 	int x, y, dist;
 	if (a->label != b->label) return FALSE;
 	x = a->priority;
@@ -116,13 +118,14 @@ int match(Connector *a, Connector *b, int aw, int bw) {
 	} else return FALSE;
 }
 
-typedef struct Table_connector Table_connector;
-struct Table_connector {
-	short			  lw, rw;
-	Connector		 *le, *re;
-	short			  cost;
-	int				count;
-	Table_connector   *next;
+typedef struct Table_connector_s Table_connector;
+struct Table_connector_s
+{
+	short            lw, rw;
+	Connector        *le, *re;
+	short            cost;
+	s64              count;
+	Table_connector  *next;
 };
 
 static int table_size;
@@ -173,8 +176,13 @@ void free_table(Sentence sent) {
 	xfree((void *) table, table_size * sizeof(Table_connector*));
 }
 
-static Table_connector * table_store(int lw, int rw, Connector *le, Connector *re, int cost, int count){
-	/* Stores the value in the table.  Assumes it's not already there */
+/** 
+ * Stores the value in the table.  Assumes it's not already there.
+ */
+static Table_connector * table_store(int lw, int rw,
+                                     Connector *le, Connector *re,
+                                     int cost, s64 count)
+{
 	Table_connector *t, *n;
 	int h;
 
@@ -188,9 +196,11 @@ static Table_connector * table_store(int lw, int rw, Connector *le, Connector *r
 	return n;
 }
 
-
-static Table_connector * table_pointer(int lw, int rw, Connector *le, Connector *re, int cost) {
-	/* returns the pointer to this info, NULL if not there */
+/** returns the pointer to this info, NULL if not there */
+static Table_connector * find_table_pointer(int lw, int rw, 
+                                       Connector *le, Connector *re,
+                                       int cost)
+{
 	Table_connector *t;
 	t = table[hash(lw, rw, le, re, cost)];
 	for (; t != NULL; t = t->next) {
@@ -198,15 +208,19 @@ static Table_connector * table_pointer(int lw, int rw, Connector *le, Connector
 			&& (t->cost == cost))  return t;
 	}
 
-	if ((current_resources != NULL) &&		 resources_exhausted(current_resources)) {
+	/* Create a new connector only if resources are exhausted.
+	 * (???) Huh? I guess we're in panic parse mode in that case.
+	 */
+	if ((current_resources != NULL) && resources_exhausted(current_resources)) {
 		return table_store(lw, rw, le, re, cost, 0);
 	}
 	else return NULL;
 }
 
-int table_lookup(int lw, int rw, Connector *le, Connector *re, int cost) {
-	/* returns the count for this quintuple if there, -1 otherwise */
-	Table_connector *t = table_pointer(lw, rw, le, re, cost);
+/** returns the count for this quintuple if there, -1 otherwise */
+s64 table_lookup(int lw, int rw, Connector *le, Connector *re, int cost)
+{
+	Table_connector *t = find_table_pointer(lw, rw, le, re, cost);
 
 	if (t == NULL) return -1; else return t->count;
 }
@@ -217,9 +231,9 @@ int table_lookup(int lw, int rw, Connector *le, Connector *re, int cost) {
  */
 static void table_update(int lw, int rw, 
                          Connector *le, Connector *re,
-                         int cost, int count)
+                         int cost, s64 count)
 {
-	Table_connector *t = table_pointer(lw, rw, le, re, cost);
+	Table_connector *t = find_table_pointer(lw, rw, le, re, cost);
 
 	assert(t != NULL, "This entry is supposed to be in the table.");
 	t->count = count;
@@ -229,19 +243,19 @@ static void table_update(int lw, int rw,
  * Returns 0 if and only if this entry is in the hash table 
  * with a count value of 0.
  */
-static int pseudocount(int lw, int rw, Connector *le, Connector *re, int cost)
+static s64 pseudocount(int lw, int rw, Connector *le, Connector *re, int cost)
 {
-	int count;
+	s64 count;
 	count = table_lookup(lw, rw, le, re, cost);
 	if (count == 0) return 0; else return 1;
 }
 
-int count(int lw, int rw, Connector *le, Connector *re, int cost)
+static s64 count(int lw, int rw, Connector *le, Connector *re, int cost)
 {
 	Disjunct * d;
-	int total, pseudototal;
+	s64 total, pseudototal;
 	int start_word, end_word, w;
-	int leftcount, rightcount;
+	s64 leftcount, rightcount;
 	int lcost, rcost, Lmatch, Rmatch;
 
 	Match_node * m, *m1;
@@ -249,11 +263,12 @@ int count(int lw, int rw, Connector *le, Connector *re, int cost)
 
 	if (cost < 0) return 0;  /* will we ever call it with cost<0 ? */
 
-	t = table_pointer(lw, rw, le, re, cost);
+	t = find_table_pointer(lw, rw, le, re, cost);
 
 	if (t == NULL) {
-		t = table_store(lw, rw, le, re, cost, 0);  /* create the table entry with a tentative cost of 0 */
-												   /* this cost must be updated before we return */
+		/* Create the table entry with a tentative cost of 0. 
+	    * This cost must be updated before we return. */
+		t = table_store(lw, rw, le, re, cost, 0);
 	} else {
 		return t->count;
 	}
@@ -316,7 +331,7 @@ int count(int lw, int rw, Connector *le, Connector *re, int cost)
 
 	total = 0;
 
-	for (w=start_word; w <= end_word; w++) {
+	for (w=start_word; w < end_word+1; w++) {
 		m1 = m = form_match_list(w, le, lw, re, rw);
 		for (; m!=NULL; m=m->next) {
 			d = m->d;
@@ -398,9 +413,9 @@ int count(int lw, int rw, Connector *le, Connector *re, int cost)
  * specified cost Assumes that the hash table has already been
  * initialized, and is freed later.
  */
-int parse(Sentence sent, int cost, Parse_Options opts)
+s64 parse(Sentence sent, int cost, Parse_Options opts)
 {
-	int total;
+	s64 total;
 
 	count_set_effective_distance(sent);
 	current_resources = opts->resources;
@@ -411,10 +426,10 @@ int parse(Sentence sent, int cost, Parse_Options opts)
 
 	total = count(-1, sent->length, NULL, NULL, cost+1);
 	if (verbosity > 1) {
-		printf("Total count with %d null links:   %d\n", cost, total);
+		printf("Total count with %d null links:   %lld\n", cost, total);
 	}
-	if (total < 0) {
-		printf("WARNING: Overflow in count!\n");
+	if ((verbosity > 0) && (PARSE_NUM_OVERFLOW < total)) {
+		printf("WARNING: Overflow in count! cnt=%lld\n", total);
 	}
 
 	local_sent = NULL;
@@ -454,11 +469,13 @@ int parse(Sentence sent, int cost, Parse_Options opts)
    2  This region can be completed, and it's been marked.
    */
 
-static int region_valid(int lw, int rw, Connector *le, Connector *re) {
-	/* Returns 0 if this range cannot be successfully filled in with		   */
-	/* links.  Returns 1 if it can, and it's not been marked, and returns	  */
-	/* 2 if it can and it has been marked.									 */
-
+/**
+ * Returns 0 if this range cannot be successfully filled in with
+ * links.  Returns 1 if it can, and it's not been marked, and returns
+ * 2 if it can and it has been marked.
+ */
+static int region_valid(int lw, int rw, Connector *le, Connector *re)
+{
 	Disjunct * d;
 	int left_valid, right_valid, found;
 	int i, start_word, end_word;
@@ -486,7 +503,7 @@ static int region_valid(int lw, int rw, Connector *le, Connector *re) {
 
 	found = 0;
 
-	for (w=start_word; w <= end_word; w++) {
+	for (w=start_word; w < end_word+1; w++) {
 		m1 = m = form_match_list(w, le, lw, re, rw);
 		for (; m!=NULL; m=m->next) {
 			d = m->d;
@@ -519,11 +536,15 @@ static int region_valid(int lw, int rw, Connector *le, Connector *re) {
 	return found;
 }
 
-static void mark_region(int lw, int rw, Connector *le, Connector *re) {
-	/* Mark as useful all disjuncts involved in some way to complete the structure  */
-	/* within the current region.  Note that only disjuncts strictly between		*/
-	/* lw and rw will be marked.  If it so happens that this region itself is not   */
-	/* valid, then this fact will be recorded in the table, and nothing else happens*/
+/**
+ * Mark as useful all disjuncts involved in some way to complete the
+ * structure within the current region.  Note that only disjuncts
+ * strictly between lw and rw will be marked.  If it so happens that
+ * this region itself is not valid, then this fact will be recorded
+ * in the table, and nothing else happens.
+ */
+static void mark_region(int lw, int rw, Connector *le, Connector *re)
+{
 
 	Disjunct * d;
 	int left_valid, right_valid, i;
@@ -559,7 +580,7 @@ static void mark_region(int lw, int rw, Connector *le, Connector *re) {
 		end_word = re->word;
 	}
 
-	for (w=start_word; w <= end_word; w++) {
+	for (w=start_word; w < end_word+1; w++) {
 		m1 = m = form_match_list(w, le, lw, re, rw);
 		for (; m!=NULL; m=m->next) {
 			d = m->d;
@@ -633,18 +654,18 @@ void delete_unmarked_disjuncts(Sentence sent){
 	}
 }
 
-void conjunction_prune(Sentence sent, Parse_Options opts) {
-	/*
-	   We've already built the sentence disjuncts, and we've pruned them
-	   and power_pruned(GENTLE) them also.  The sentence contains a
-	   conjunction.  deletable[][] has been initialized to indicate the
-	   ranges which may be deleted in the final linkage.
-	
-	   This routine deletes irrelevant disjuncts.  It finds them by first
-	   marking them all as irrelevant, and then marking the ones that
-	   might be useable.  Finally, the unmarked ones are removed.
-
-	   */
+/**
+ * We've already built the sentence disjuncts, and we've pruned them
+ * and power_pruned(GENTLE) them also.  The sentence contains a
+ * conjunction.  deletable[][] has been initialized to indicate the
+ * ranges which may be deleted in the final linkage.
+ *
+ * This routine deletes irrelevant disjuncts.  It finds them by first
+ * marking them all as irrelevant, and then marking the ones that
+ * might be useable.  Finally, the unmarked ones are removed.
+ */
+void conjunction_prune(Sentence sent, Parse_Options opts)
+{
 	Disjunct * d;
 	int w;
 
@@ -652,7 +673,7 @@ void conjunction_prune(Sentence sent, Parse_Options opts) {
 	deletable = sent->deletable;
 	count_set_effective_distance(sent);
 
-	/* we begin by unmarking all disjuncts.  This would not be necessary if
+	/* We begin by unmarking all disjuncts.  This would not be necessary if
 	   whenever we created a disjunct we cleared its marked field.
 	   I didn't want to search the program for all such places, so
 	   I did this way.
