@@ -36,6 +36,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <wchar.h>
 
 #include <link-grammar/link-includes.h>
 #include <link-grammar/structures.h>
@@ -53,25 +54,36 @@ static Parse_Options  opts;
 static Parse_Options  panic_parse_opts;
 static int verbosity = 0;
 
-typedef enum {UNGRAMMATICAL='*',
-			  PARSE_WITH_DISJUNCT_COST_GT_0=':',
-			  NO_LABEL=' '} Label;
+typedef enum
+{
+	UNGRAMMATICAL='*',
+	PARSE_WITH_DISJUNCT_COST_GT_0=':',
+	NO_LABEL=' '
+} Label;
 
-static int fget_input_string(char *input_string, FILE *in, FILE *out,
-							 Parse_Options opts) {
-	if ((!parse_options_get_batch_mode(opts)) && (verbosity > 0)
-		&& (!input_pending)) fprintf(out, "linkparser> ");
-	fflush(out);
+static wint_t 
+fget_input_string(char *input_string, size_t maxlen,
+                  FILE *in, FILE *out, Parse_Options opts)
+{
+	if ((!parse_options_get_batch_mode(opts)) && 
+	    (verbosity > 0) && 
+	    (!input_pending))
+	{
+		fprintf(out, "linkparser> ");
+		fflush(out);
+	}
 	input_pending = FALSE;
-	if (fgets(input_string, MAXINPUT, in)) return 1;
+
+	if (fgets(input_string, maxlen, in)) return 1;
 	else return 0;
 }
 
-static int fget_input_char(FILE *in, FILE *out, Parse_Options opts) {
+static wint_t fget_input_char(FILE *in, FILE *out, Parse_Options opts)
+{
 	if (!parse_options_get_batch_mode(opts) && (verbosity > 0))
-	  fprintf(out, "linkparser> ");
+		fprintf(out, "linkparser> ");
 	fflush(out);
-	return getc(in);
+	return fgetwc(in);
 }
 
 /**************************************************************************
@@ -82,9 +94,13 @@ static int fget_input_char(FILE *in, FILE *out, Parse_Options opts) {
 *
 **************************************************************************/
 
-static void process_linkage(Linkage linkage, Parse_Options opts) {
+static void process_linkage(Linkage linkage, Parse_Options opts)
+{
 	char * string;
-	int	j, mode, first_sublinkage;
+	int j, mode, first_sublinkage;
+	int nlink;
+
+	if (!linkage) return;  /* Can happen in timeout mode */
 
 	if (parse_options_get_display_union(opts)) {
 		linkage_compute_union(linkage);
@@ -94,7 +110,9 @@ static void process_linkage(Linkage linkage, Parse_Options opts) {
 		first_sublinkage = 0;
 	}
 
-	for (j=first_sublinkage; j<linkage_get_num_sublinkages(linkage); ++j) {
+	nlink = linkage_get_num_sublinkages(linkage);
+	for (j=first_sublinkage; j<nlink; ++j)
+	{
 		linkage_set_current_sublinkage(linkage, j);
 		if (parse_options_get_display_on(opts)) {
 			string = linkage_print_diagram(linkage);
@@ -149,8 +167,10 @@ static void print_parse_statistics(Sentence sent, Parse_Options opts) {
 }
 
 
-static void process_some_linkages(Sentence sent, Parse_Options opts) {
-	int i, c, num_displayed, num_to_query;
+static void process_some_linkages(Sentence sent, Parse_Options opts)
+{
+	wint_t c;
+	int i, num_displayed, num_to_query;
 	Linkage linkage;
 
 	if (verbosity > 0) print_parse_statistics(sent, opts);
@@ -159,7 +179,7 @@ static void process_some_linkages(Sentence sent, Parse_Options opts) {
 	}
 	else {
 		num_to_query = MIN(sentence_num_linkages_post_processed(sent),
-						   DISPLAY_MAX);
+		                   DISPLAY_MAX);
 	}
 
 	for (i=0, num_displayed=0; i<num_to_query; ++i) {
@@ -172,33 +192,36 @@ static void process_some_linkages(Sentence sent, Parse_Options opts) {
 		linkage = linkage_create(i, sent, opts);
 
 		if (verbosity > 0) {
-		  if ((sentence_num_valid_linkages(sent) == 1) &&
-			  (!parse_options_get_display_bad(opts))) {
-			fprintf(stdout, "  Unique linkage, ");
-		  }
-		  else if ((parse_options_get_display_bad(opts)) &&
-				   (sentence_num_violations(sent, i) > 0)) {
-			fprintf(stdout, "  Linkage %d (bad), ", i+1);
-		  }
-		  else {
-			fprintf(stdout, "  Linkage %d, ", i+1);
-		  }
+			if ((sentence_num_valid_linkages(sent) == 1) &&
+				(!parse_options_get_display_bad(opts)))
+			{
+				fprintf(stdout, "	Unique linkage, ");
+			}
+			else if ((parse_options_get_display_bad(opts)) &&
+			         (sentence_num_violations(sent, i) > 0))
+			{
+				fprintf(stdout, "	Linkage %d (bad), ", i+1);
+			}
+			else
+			{
+				fprintf(stdout, "	Linkage %d, ", i+1);
+			}
 
-		  if (!linkage_is_canonical(linkage)) {
-			fprintf(stdout, "non-canonical, ");
-		  }
-		  if (linkage_is_improper(linkage)) {
-			fprintf(stdout, "improper fat linkage, ");
-		  }
-		  if (linkage_has_inconsistent_domains(linkage)) {
-			fprintf(stdout, "inconsistent domains, ");
-		  }
+			if (!linkage_is_canonical(linkage)) {
+				fprintf(stdout, "non-canonical, ");
+			}
+			if (linkage_is_improper(linkage)) {
+				fprintf(stdout, "improper fat linkage, ");
+			}
+			if (linkage_has_inconsistent_domains(linkage)) {
+				fprintf(stdout, "inconsistent domains, ");
+			}
 
-		  fprintf(stdout, "cost vector = (UNUSED=%d DIS=%d AND=%d LEN=%d)\n",
-				  linkage_unused_word_cost(linkage),
-				  linkage_disjunct_cost(linkage),
-				  linkage_and_cost(linkage),
-				  linkage_link_cost(linkage));
+			fprintf(stdout, "cost vector = (UNUSED=%d DIS=%d AND=%d LEN=%d)\n",
+			       linkage_unused_word_cost(linkage),
+			       linkage_disjunct_cost(linkage),
+			       linkage_and_cost(linkage),
+			       linkage_link_cost(linkage));
 		}
 
 		process_linkage(linkage, opts);
@@ -208,8 +231,9 @@ static void process_some_linkages(Sentence sent, Parse_Options opts) {
 			if (verbosity > 0) {
 				fprintf(stdout, "Press RETURN for the next linkage.\n");
 			}
-			if ((c=fget_input_char(stdin, stdout, opts)) != '\n') {
-				ungetc(c, stdin);
+			if ((c = fget_input_char(stdin, stdout, opts)) != '\n')
+			{
+				ungetwc(c, stdin);
 				input_pending = TRUE;
 				break;
 			}
@@ -230,7 +254,7 @@ static int there_was_an_error(Label label, Sentence sent, Parse_Options opts) {
 			return PARSE_WITH_DISJUNCT_COST_GT_0;
 		}
 	} else {
-		if  (label != UNGRAMMATICAL) {
+		if (label != UNGRAMMATICAL) {
 			batch_errors++;
 			return UNGRAMMATICAL;
 		}
@@ -238,7 +262,10 @@ static int there_was_an_error(Label label, Sentence sent, Parse_Options opts) {
 	return FALSE;
 }
 
-static void batch_process_some_linkages(Label label, Sentence sent, Parse_Options opts) {
+static void batch_process_some_linkages(Label label, 
+                                        Sentence sent,
+                                        Parse_Options opts)
+{
 	Linkage linkage;
 
 	if (there_was_an_error(label, sent, opts)) {
@@ -251,8 +278,8 @@ static void batch_process_some_linkages(Label label, Sentence sent, Parse_Option
 	}
 }
 
-static int special_command(char *input_string, Dictionary dict) {
-
+static int special_command(char *input_string, Dictionary dict)
+{
 	if (input_string[0] == '\n') return TRUE;
 	if (input_string[0] == COMMENT_CHAR) return TRUE;
 	if (input_string[0] == '!') {
@@ -300,11 +327,11 @@ static void print_usage(char *str) {
 	exit(-1);
 }
 
-int main(int argc, char * argv[]) {
-
+int main(int argc, char * argv[])
+{
 	Dictionary      dict;
 	Sentence        sent;
-	char            *language="en";  /* default to english, and not locale */
+	const char     *language="en";  /* default to english, and not locale */
 	int             pp_on=TRUE;
 	int             af_on=TRUE;
 	int             cons_on=TRUE;
@@ -356,9 +383,9 @@ int main(int argc, char * argv[]) {
 	parse_options_set_short_length(opts, 10);
 
 	if(language && *language)
-	  dict = dictionary_create_lang(language);
+		dict = dictionary_create_lang(language);
 	else
-	  dict = dictionary_create_default_lang();
+		dict = dictionary_create_default_lang();
 
 	if (dict == NULL) {
 		fprintf(stderr, "%s\n", lperrmsg);
@@ -369,18 +396,21 @@ int main(int argc, char * argv[]) {
 	for (i=1; i<argc; i++) {
 		if ((strcmp("-pp", argv[i])==0) ||
 			(strcmp("-c", argv[i])==0) ||
-			(strcmp("-a", argv[i])==0)) {
-		  i++;
-		} else if ((argv[i][0] == '-') && (strcmp("-ppoff", argv[i])!=0) &&
-				   (argv[i][0] == '-') && (strcmp("-coff", argv[i])!=0) &&
-				   (argv[i][0] == '-') && (strcmp("-aoff", argv[i])!=0)) {
-		  issue_special_command(argv[i]+1, opts, dict);
+			(strcmp("-a", argv[i])==0))
+		{
+			i++;
+		}
+		else if ((argv[i][0] == '-') && (strcmp("-ppoff", argv[i])!=0) &&
+		         (argv[i][0] == '-') && (strcmp("-coff", argv[i])!=0) &&
+		         (argv[i][0] == '-') && (strcmp("-aoff", argv[i])!=0))
+		{
+			issue_special_command(argv[i]+1, opts, dict);
 		}
 	}
 
 	verbosity = parse_options_get_verbosity(opts);
 
-	while (fget_input_string(input_string, stdin, stdout, opts)) {
+	while (fget_input_string(input_string, MAXINPUT, stdin, stdout, opts)) {
 
 		if ((strcmp(input_string, "quit\n")==0) ||
 			(strcmp(input_string, "exit\n")==0)) break;
@@ -403,8 +433,8 @@ int main(int argc, char * argv[]) {
 		}
 		if (sentence_length(sent) > parse_options_get_max_sentence_length(opts)) {
 			if (verbosity > 0) {
-			  fprintf(stdout,
-					  "Sentence length (%d words) exceeds maximum allowable (%d words)\n",
+				fprintf(stdout,
+				       "Sentence length (%d words) exceeds maximum allowable (%d words)\n",
 					sentence_length(sent), parse_options_get_max_sentence_length(opts));
 			}
 			sentence_delete(sent);
