@@ -10,6 +10,7 @@
 
 #include <jni.h>
 #include <stdio.h>
+#include <utilities.h>
 
 #include <link-grammar/api.h>
 #include "jni-client.h"
@@ -105,14 +106,17 @@ static void jParse(char* inputString)
 	int jverbosity = parse_options_get_verbosity(opts);
 	if (sent)
 		sentence_delete(sent);
+
 	sent = sentence_create(inputString, dict);
 	num_linkages=0;
 
 	if (sent == NULL)
 		return;
 
-	if (sentence_length(sent) > parse_options_get_max_sentence_length(opts)) {
+	if (sentence_length(sent) > parse_options_get_max_sentence_length(opts))
+	{
 		sentence_delete(sent);
+		sent = NULL;
 		if (jverbosity > 0) {
 			fprintf(stdout,
 				"Sentence length (%d words) exceeds maximum allowable (%d words)\n",
@@ -148,7 +152,8 @@ static void jParse(char* inputString)
 
 	if ((num_linkages == 0) &&
 			parse_options_resources_exhausted(opts) &&
-			parse_options_get_panic_mode(opts)) {
+			parse_options_get_panic_mode(opts))
+	{
 		print_total_time(opts);
 		if (jverbosity > 0) fprintf(stdout, "Entering \"panic\" mode...\n");
 		parse_options_reset_resources(panic_parse_opts);
@@ -182,14 +187,26 @@ static void finish(void)
 {
 	if (sent)
 		sentence_delete(sent);
+	sent = NULL;
+
 #if DO_PHRASE_TREE
 	if (tree)
 		linkage_free_constituent_tree(tree);
+	tree = NULL;
 #endif
+
 	if (linkage)
 		linkage_delete(linkage);
+	linkage = NULL;
+
 	dictionary_delete(dict);
+	dict = NULL;
+
 	parse_options_delete(opts);
+	opts = NULL;
+
+	parse_options_delete(panic_parse_opts);
+	panic_parse_opts = NULL;
 }
 
 /* =========================================================================== */
@@ -207,6 +224,15 @@ Java_org_linkgrammar_LinkGrammar_setMaxCost(JNIEnv *env, jclass cls, jint maxCos
 	parse_options_set_disjunct_cost(opts, maxCost);
 }
 
+JNIEXPORT void JNICALL 
+Java_org_linkgrammar_LinkGrammar_setDictionariesPath
+  (JNIEnv *env, jclass cls, jstring path)
+{
+    const char *nativePath = (*env)->GetStringUTFChars(env,path, 0);
+    set_data_dir(nativePath);
+    (*env)->ReleaseStringUTFChars(env,path, nativePath);
+}
+
 /*
  * Class:		 LinkGrammar
  * Method:		init
@@ -366,7 +392,7 @@ JNIEXPORT jstring JNICALL
 Java_org_linkgrammar_LinkGrammar_getLinkLLabel(JNIEnv *env, jclass cls, jint i)
 {
  	/* Does not need to be freed, points into linkage */
-	char *s = linkage_get_link_llabel(linkage, i);
+	const char *s = linkage_get_link_llabel(linkage, i);
 	jstring j = (*env)->NewStringUTF(env, s);
 	return j;
 }
@@ -380,7 +406,7 @@ JNIEXPORT jstring JNICALL
 Java_org_linkgrammar_LinkGrammar_getLinkRLabel(JNIEnv *env, jclass cls, jint i)
 {
  	/* Does not need to be freed, points into linkage */
-	char *s = linkage_get_link_rlabel(linkage, i);
+	const char *s = linkage_get_link_rlabel(linkage, i);
 	jstring j = (*env)->NewStringUTF(env, s);
 	return j;
 }
@@ -394,7 +420,7 @@ JNIEXPORT jstring JNICALL
 Java_org_linkgrammar_LinkGrammar_getLinkLabel(JNIEnv *env, jclass cls, jint i)
 {
  	/* Does not need to be freed, points into linkage */
-	char *s = linkage_get_link_label(linkage, i);
+	const char *s = linkage_get_link_label(linkage, i);
 	jstring j = (*env)->NewStringUTF(env, s);
 	return j;
 }
