@@ -25,6 +25,15 @@ JNIEXPORT void JNICALL Java_org_linkgrammar_LinkGrammar_setMaxParseSeconds
 JNIEXPORT void JNICALL Java_org_linkgrammar_LinkGrammar_setMaxCost
 	(JNIEnv *, jclass, jint);
 
+/*
+ * Class:     LinkGrammar
+ * Method:    setDictionariesPath
+ * Signature: (Ljava/lang/String;)V
+ */
+JNIEXPORT void JNICALL Java_org_linkgrammar_LinkGrammar_setDictionariesPath
+  (JNIEnv *, jclass, jstring);
+
+
 /*
  * Class:     LinkGrammar
  * Method:    init
