@@ -28,7 +28,7 @@ typedef struct Dictionary_s * Dictionary;
 link_public_api(Dictionary) 
      dictionary_create(char * dict_name, char * pp_name, char * cons_name, char * affix_name);
 link_public_api(Dictionary)
-     dictionary_create_lang(char * lang);
+     dictionary_create_lang(const char * lang);
 link_public_api(Dictionary)
      dictionary_create_default_lang(void);
 
@@ -236,19 +236,19 @@ link_public_api(int)
      linkage_get_link_rword(Linkage linkage, int index);
 link_public_api(int)
      linkage_get_link_length(Linkage linkage, int index);
-link_public_api(char *)
+link_public_api(const char *)
      linkage_get_link_label(Linkage linkage, int index);
-link_public_api(char *)
+link_public_api(const char *)
      linkage_get_link_llabel(Linkage linkage, int index);
-link_public_api(char *)
+link_public_api(const char *)
      linkage_get_link_rlabel(Linkage linkage, int index);
 link_public_api(int)
      linkage_get_link_num_domains(Linkage linkage, int index);
 link_public_api(char **)
      linkage_get_link_domain_names(Linkage linkage, int index);
-link_public_api(char **)
+link_public_api(const char **)
      linkage_get_words(Linkage linkage);
-link_public_api(char *)
+link_public_api(const char *)
      linkage_get_word(Linkage linkage, int w);
 link_public_api(char *)
      linkage_print_links_and_domains(Linkage linkage);
@@ -280,7 +280,7 @@ link_public_api(int)
      linkage_is_improper(Linkage linkage);
 link_public_api(int)
      linkage_has_inconsistent_domains(Linkage linkage);
-link_public_api(char *)
+link_public_api(const char *)
      linkage_get_violation_name(Linkage linkage);
 
 
