@@ -17,8 +17,8 @@
 
 static int center[MAX_SENTENCE];
 static int N_words_to_print;  /* version of N_words in this file for printing links */
-char * trailer(int mode);
-char * header(int mode);
+const char * trailer(int mode);
+const char * header(int mode);
 
 static void set_centers(Linkage linkage, int print_word_0) {
     int i, len, tot;
@@ -57,11 +57,12 @@ static void left_append_string(String * string, const char * s, const char * t)
 
 
 
-static void print_a_link(String * s, Linkage linkage, int link) {
+static void print_a_link(String * s, Linkage linkage, int link)
+{
     Sentence sent = linkage_get_sentence(linkage);
     Dictionary dict = sent->dict;
     int l, r;
-    char *label, *llabel, *rlabel;
+    const char *label, *llabel, *rlabel;
     
     l      = linkage_get_link_lword(linkage, link);
     r      = linkage_get_link_rword(linkage, link);
@@ -70,11 +71,11 @@ static void print_a_link(String * s, Linkage linkage, int link) {
     rlabel = linkage_get_link_rlabel(linkage, link);
 
     if ((l == 0) && dict->left_wall_defined) {
-	left_append_string(s, LEFT_WALL_DISPLAY,"               ");
+        left_append_string(s, LEFT_WALL_DISPLAY,"               ");
     } else if ((l == (linkage_get_num_words(linkage)-1)) && dict->right_wall_defined) {
-	left_append_string(s, RIGHT_WALL_DISPLAY,"               ");	
+        left_append_string(s, RIGHT_WALL_DISPLAY,"               ");	
     } else {
-	left_append_string(s, linkage_get_word(linkage, l), "               ");
+        left_append_string(s, linkage_get_word(linkage, l), "               ");
     }
     left_append_string(s, llabel, "     ");
     append_string(s, "   <---"); 
@@ -228,54 +229,57 @@ static char * build_linkage_postscript_string(Linkage linkage) {
     return ps_string; 
 }
 
-void compute_chosen_words(Sentence sent, Linkage linkage) {
-    /*
-       This takes the current chosen_disjuncts array and uses it to
-       compute the chosen_words array.  "I.xx" suffixes are eliminated.
-       */
+/**
+ * This takes the current chosen_disjuncts array and uses it to
+ * compute the chosen_words array.  "I.xx" suffixes are eliminated.
+ */
+void compute_chosen_words(Sentence sent, Linkage linkage)
+{
     int i, l;
-    char * s, *t, *u;
+    const char *t;
+    char * s, *u;
     Parse_info pi = sent->parse_info;
-    char * chosen_words[MAX_SENTENCE];
+    const char * chosen_words[MAX_SENTENCE];
     Parse_Options opts = linkage->opts;
 
     for (i=0; i<sent->length; i++) {   /* get rid of those ugly ".Ixx" */
-	chosen_words[i] = sent->word[i].string;
-	if (pi->chosen_disjuncts[i] == NULL) {  
-	    /* no disjunct is used on this word because of null-links */
-	    t = chosen_words[i];
-	    l = strlen(t) + 2;
-	    s = (char *) xalloc(l+1);
-	    sprintf(s, "[%s]", t);
-	    t = string_set_add(s, sent->string_set);
-	    xfree(s, l+1);
-	    chosen_words[i] = t;
-	} else if (opts->display_word_subscripts) {
-	    t = pi->chosen_disjuncts[i]->string;	
-	    if (is_idiom_word(t)) {
-		l = strlen(t);
-		s = (char *) xalloc(l+1);
-		strcpy(s,t);
-		for (u=s; *u != '.'; u++)
-		  ;
-		*u = '\0';
-		t = string_set_add(s, sent->string_set);
-		xfree(s, l+1);
-		chosen_words[i] = t;
-	    } else {
-		chosen_words[i] = t;
-	    }
-	}
+        chosen_words[i] = sent->word[i].string;
+        if (pi->chosen_disjuncts[i] == NULL) {  
+            /* no disjunct is used on this word because of null-links */
+            t = chosen_words[i];
+            l = strlen(t) + 2;
+            s = (char *) xalloc(l+1);
+            sprintf(s, "[%s]", t);
+            t = string_set_add(s, sent->string_set);
+            xfree(s, l+1);
+            chosen_words[i] = t;
+        } else if (opts->display_word_subscripts) {
+            t = pi->chosen_disjuncts[i]->string;        
+            if (is_idiom_word(t)) {
+                l = strlen(t);
+                s = (char *) xalloc(l+1);
+                strcpy(s,t);
+                for (u=s; *u != '.'; u++)
+                  ;
+                *u = '\0';
+                t = string_set_add(s, sent->string_set);
+                xfree(s, l+1);
+                chosen_words[i] = t;
+            } else {
+                chosen_words[i] = t;
+            }
+        }
     }
     if (sent->dict->left_wall_defined) {
-	chosen_words[0] = LEFT_WALL_DISPLAY;
+        chosen_words[0] = LEFT_WALL_DISPLAY;
     }
     if (sent->dict->right_wall_defined) {
-	chosen_words[sent->length-1] = RIGHT_WALL_DISPLAY;
+        chosen_words[sent->length-1] = RIGHT_WALL_DISPLAY;
     }
     for (i=0; i<linkage->num_words; ++i) {
-	linkage->word[i] = (char *) exalloc(strlen(chosen_words[i])+1);
-	strcpy(linkage->word[i], chosen_words[i]);
+        s = (char *) exalloc(strlen(chosen_words[i])+1);
+        strcpy(s, chosen_words[i]);
+        linkage->word[i] = s;
     }
 }
 
@@ -292,7 +296,8 @@ static char xpicture[MAX_HEIGHT][MAX_LINE];
 char * linkage_print_diagram(Linkage linkage)
 {
     int i, j, k, cl, cr, row, top_row, width, flag;
-    char *t, *s;
+    const char *s;
+    char *t;
     int print_word_0 = 0, print_word_N = 0, N_wall_connectors, suppressor_used;
     char connector[MAX_TOKEN_LENGTH];
     int line_len, link_length;
@@ -309,40 +314,40 @@ char * linkage_print_diagram(Linkage linkage)
 
     N_wall_connectors = 0;
     if (dict->left_wall_defined) {
-	suppressor_used = FALSE;
-	if (!opts->display_walls) 
-	    for (j=0; j<N_links; j++) {
-		if ((ppla[j]->l == 0)) {
-		    if (ppla[j]->r == linkage->num_words-1) continue;
-		    N_wall_connectors ++;
-		    if (strcmp(ppla[j]->lc->string, LEFT_WALL_SUPPRESS)==0){
-			suppressor_used = TRUE;
-		    }
-		}
-	    }
-	print_word_0 = (((!suppressor_used) && (N_wall_connectors != 0)) 
-			|| (N_wall_connectors > 1) || opts->display_walls);
+        suppressor_used = FALSE;
+        if (!opts->display_walls) 
+            for (j=0; j<N_links; j++) {
+                if ((ppla[j]->l == 0)) {
+                    if (ppla[j]->r == linkage->num_words-1) continue;
+                    N_wall_connectors ++;
+                    if (strcmp(ppla[j]->lc->string, LEFT_WALL_SUPPRESS)==0){
+                        suppressor_used = TRUE;
+                    }
+                }
+            }
+        print_word_0 = (((!suppressor_used) && (N_wall_connectors != 0)) 
+                        || (N_wall_connectors > 1) || opts->display_walls);
     } 
     else {
-	print_word_0 = TRUE;
+        print_word_0 = TRUE;
     }
 
     N_wall_connectors = 0;
     if (dict->right_wall_defined) {
-	suppressor_used = FALSE;
-	for (j=0; j<N_links; j++) {
-	    if (ppla[j]->r == linkage->num_words-1) {
-		N_wall_connectors ++;
-		if (strcmp(ppla[j]->lc->string, RIGHT_WALL_SUPPRESS)==0){
-		    suppressor_used = TRUE;
-		}
-	    }
-	}
-	print_word_N = (((!suppressor_used) && (N_wall_connectors != 0)) 
-			|| (N_wall_connectors > 1) || opts->display_walls);
+        suppressor_used = FALSE;
+        for (j=0; j<N_links; j++) {
+            if (ppla[j]->r == linkage->num_words-1) {
+                N_wall_connectors ++;
+                if (strcmp(ppla[j]->lc->string, RIGHT_WALL_SUPPRESS)==0){
+                    suppressor_used = TRUE;
+                }
+            }
+        }
+        print_word_N = (((!suppressor_used) && (N_wall_connectors != 0)) 
+                        || (N_wall_connectors > 1) || opts->display_walls);
     } 
     else {
-	print_word_N = TRUE;
+        print_word_N = TRUE;
     }
 
     N_words_to_print = linkage->num_words;
@@ -352,130 +357,133 @@ char * linkage_print_diagram(Linkage linkage)
     line_len = center[N_words_to_print-1]+1;
     
     for (k=0; k<MAX_HEIGHT; k++) {
-	for (j=0; j<line_len; j++) picture[k][j] = ' ';
-	picture[k][line_len] = '\0';
+        for (j=0; j<line_len; j++) picture[k][j] = ' ';
+        picture[k][line_len] = '\0';
     }
     top_row = 0;
     
     for (link_length = 1; link_length < N_words_to_print; link_length++) {
-	for (j=0; j<N_links; j++) {
-	    if (ppla[j]->l == -1) continue;
-	    if ((ppla[j]->r - ppla[j]->l) != link_length)
-	      continue;
-	    if (!print_word_0 && (ppla[j]->l == 0)) continue;
-	    /* gets rid of the irrelevant link to the left wall */
-	    if (!print_word_N && (ppla[j]->r == linkage->num_words-1)) continue;	    
-	    /* gets rid of the irrelevant link to the right wall */
+        for (j=0; j<N_links; j++) {
+            if (ppla[j]->l == -1) continue;
+            if ((ppla[j]->r - ppla[j]->l) != link_length)
+              continue;
+            if (!print_word_0 && (ppla[j]->l == 0)) continue;
+            /* gets rid of the irrelevant link to the left wall */
+            if (!print_word_N && (ppla[j]->r == linkage->num_words-1)) continue;            
+            /* gets rid of the irrelevant link to the right wall */
 
-	    /* put it into the lowest position */
-	    cl = center[ppla[j]->l];
-	    cr = center[ppla[j]->r];
-	    for (row=0; row < MAX_HEIGHT; row++) {
-		for (k=cl+1; k<cr; k++) {
-		    if (picture[row][k] != ' ') break;
-		}
-		if (k == cr) break;
-	    }
-	    /* we know it fits, so put it in this row */
+            /* put it into the lowest position */
+            cl = center[ppla[j]->l];
+            cr = center[ppla[j]->r];
+            for (row=0; row < MAX_HEIGHT; row++) {
+                for (k=cl+1; k<cr; k++) {
+                    if (picture[row][k] != ' ') break;
+                }
+                if (k == cr) break;
+            }
+            /* we know it fits, so put it in this row */
 
-	    link_heights[j] = row;
-	    
-	    if (2*row+2 > MAX_HEIGHT-1) {
-		append_string(string, "The diagram is too high.\n");
-		gr_string = exalloc(strlen(string->p)+1);
-		strcpy(gr_string, string->p);
-		exfree(string->p, sizeof(char)*string->allocated);
-		exfree(string, sizeof(String));
-		return gr_string; 
-	    }
-	    if (row > top_row) top_row = row;
-	    
-	    picture[row][cl] = '+';
-	    picture[row][cr] = '+';
-	    for (k=cl+1; k<cr; k++) {
-		picture[row][k] = '-';
-	    }
-	    s = ppla[j]->name;
-	    
-	    if (opts->display_link_subscripts) {
-	      if (!isalpha((int)*s))
-		s = "";
-	    } else {
-	      if (!isupper((int)*s)) {
-		s = "";   /* Don't print fat link connector name */
-	      }
-	    }
-	    strncpy(connector, s, MAX_TOKEN_LENGTH-1);
-	    connector[MAX_TOKEN_LENGTH-1] = '\0';
-	    k=0;
-	    if (opts->display_link_subscripts)
-	      k = strlen(connector);
-	    else
-	      for (t=connector; isupper((int)*t); t++) k++; /* uppercase len of conn*/
-	    if ((cl+cr-k)/2 + 1 <= cl) {
-		t = picture[row] + cl + 1;
-	    } else {
-		t = picture[row] + (cl+cr-k)/2 + 1;
-	    }
-	    s = connector;
-	    if (opts->display_link_subscripts)
-	      while((*s != '\0') && (*t == '-')) *t++ = *s++; 
-	    else
-	      while(isupper((int)*s) && (*t == '-')) *t++ = *s++; 
+            link_heights[j] = row;
+            
+            if (2*row+2 > MAX_HEIGHT-1) {
+                append_string(string, "The diagram is too high.\n");
+                gr_string = exalloc(strlen(string->p)+1);
+                strcpy(gr_string, string->p);
+                exfree(string->p, sizeof(char)*string->allocated);
+                exfree(string, sizeof(String));
+                return gr_string; 
+            }
+            if (row > top_row) top_row = row;
+            
+            picture[row][cl] = '+';
+            picture[row][cr] = '+';
+            for (k=cl+1; k<cr; k++) {
+                picture[row][k] = '-';
+            }
+            s = ppla[j]->name;
+            
+            if (opts->display_link_subscripts)
+            {
+                if (!is_utf8_alpha(s))
+                  s = "";
+            }
+            else
+            {
+                if (!is_utf8_upper(s)) {
+                  s = "";   /* Don't print fat link connector name */
+                }
+            }
+            strncpy(connector, s, MAX_TOKEN_LENGTH-1);
+            connector[MAX_TOKEN_LENGTH-1] = '\0';
+            k=0;
+            if (opts->display_link_subscripts)
+                k = strlen(connector);
+            else
+                for (t=connector; isupper((int)*t); t++) k++; /* uppercase len of conn*/
+            if ((cl+cr-k)/2 + 1 <= cl) {
+                t = picture[row] + cl + 1;
+            } else {
+                t = picture[row] + (cl+cr-k)/2 + 1;
+            }
+            s = connector;
+            if (opts->display_link_subscripts)
+                while((*s != '\0') && (*t == '-')) *t++ = *s++; 
+            else
+                while(isupper((int)*s) && (*t == '-')) *t++ = *s++; 
 
-	    /* now put in the | below this one, where needed */
-	    for (k=0; k<row; k++) {
-		if (picture[k][cl] == ' ') {
-		    picture[k][cl] = '|';
-		}
-		if (picture[k][cr] == ' ') {
-		    picture[k][cr] = '|';
-		}
-	    }
-	}
+            /* now put in the | below this one, where needed */
+            for (k=0; k<row; k++) {
+                if (picture[k][cl] == ' ') {
+                    picture[k][cl] = '|';
+                }
+                if (picture[k][cr] == ' ') {
+                    picture[k][cr] = '|';
+                }
+            }
+        }
     }
     
     /* we have the link picture, now put in the words and extra "|"s */
     
-    s = xpicture[0];
+    t = xpicture[0];
     if (print_word_0) k = 0; else k = 1;
     for (; k<N_words_to_print; k++) {
-	t = linkage->word[k];
-	i=0;
-	while(*t != '\0') {
-	    *s++ = *t++;
-	    i++;
-	}
-	*s++ = ' ';
+        s = linkage->word[k];
+        i=0;
+        while(*s != '\0') {
+            *t++ = *s++;
+            i++;
+        }
+        *t++ = ' ';
     }
-    *s = '\0';
+    *t = '\0';
     
     if (opts->display_short) {
-	for (k=0; picture[0][k] != '\0'; k++) {
-	    if ((picture[0][k] == '+') || (picture[0][k] == '|')) {
-		xpicture[1][k] = '|';
-	    } else {
-		xpicture[1][k] = ' ';
-	    }
-	}
-	xpicture[1][k] = '\0';
-	for (row=0; row <= top_row; row++) {
-	    strcpy(xpicture[row+2],picture[row]);
-	}
-	top_row = top_row+2;
+        for (k=0; picture[0][k] != '\0'; k++) {
+            if ((picture[0][k] == '+') || (picture[0][k] == '|')) {
+                xpicture[1][k] = '|';
+            } else {
+                xpicture[1][k] = ' ';
+            }
+        }
+        xpicture[1][k] = '\0';
+        for (row=0; row < top_row+1; row++) {
+            strcpy(xpicture[row+2],picture[row]);
+        }
+        top_row = top_row+2;
     } else {
-	for (row=0; row <= top_row; row++) {
-	    strcpy(xpicture[2*row+2],picture[row]);
-	    for (k=0; picture[row][k] != '\0'; k++) {
-		if ((picture[row][k] == '+') || (picture[row][k] == '|')) {
-		    xpicture[2*row+1][k] = '|';
-		} else {
-		    xpicture[2*row+1][k] = ' ';
-		}
-	    }
-	    xpicture[2*row+1][k] = '\0';
-	}
-	top_row = 2*top_row + 2;
+        for (row=0; row < top_row+1; row++) {
+            strcpy(xpicture[2*row+2],picture[row]);
+            for (k=0; picture[row][k] != '\0'; k++) {
+                if ((picture[row][k] == '+') || (picture[row][k] == '|')) {
+                    xpicture[2*row+1][k] = '|';
+                } else {
+                    xpicture[2*row+1][k] = ' ';
+                }
+            }
+            xpicture[2*row+1][k] = '\0';
+        }
+        top_row = 2*top_row + 2;
     }
     
     /* we've built the picture, now print it out */
@@ -486,29 +494,29 @@ char * linkage_print_diagram(Linkage linkage)
     row_starts[N_rows] = 0;
     N_rows++;
     while(i < N_words_to_print) {
-	append_string(string, "\n");
-	width = 0;
-	do {
-	    width += strlen(linkage->word[i])+1;
-	    i++;
-	} while((i<N_words_to_print) &&
-	      (width + ((int)strlen(linkage->word[i]))+1 < x_screen_width));
-	row_starts[N_rows] = i - (!print_word_0);    /* PS junk */
-	if (i<N_words_to_print) N_rows++;     /* same */
-	for (row = top_row; row >= 0; row--) {
-	    flag = TRUE;
-	    for (j=k;flag&&(j<k+width)&&(xpicture[row][j]!='\0'); j++){
-		flag = flag && (xpicture[row][j] == ' ');
-	    }
-	    if (!flag) {
-		for (j=k;(j<k+width)&&(xpicture[row][j]!='\0'); j++){
-		    append_string(string, "%c", xpicture[row][j]);
-		}
-		append_string(string, "\n");
-	    }
-	}
-	append_string(string, "\n");
-	k += width;
+        append_string(string, "\n");
+        width = 0;
+        do {
+            width += strlen(linkage->word[i])+1;
+            i++;
+        } while((i<N_words_to_print) &&
+              (width + ((int)strlen(linkage->word[i]))+1 < x_screen_width));
+        row_starts[N_rows] = i - (!print_word_0);    /* PS junk */
+        if (i<N_words_to_print) N_rows++;     /* same */
+        for (row = top_row; row >= 0; row--) {
+            flag = TRUE;
+            for (j=k;flag&&(j<k+width)&&(xpicture[row][j]!='\0'); j++){
+                flag = flag && (xpicture[row][j] == ' ');
+            }
+            if (!flag) {
+                for (j=k;(j<k+width)&&(xpicture[row][j]!='\0'); j++){
+                    append_string(string, "%c", xpicture[row][j]);
+                }
+                append_string(string, "\n");
+            }
+        }
+        append_string(string, "\n");
+        k += width;
     }
     gr_string = exalloc(strlen(string->p)+1);
     strcpy(gr_string, string->p);
@@ -569,29 +577,34 @@ void print_expression_sizes(Sentence sent) {
     printf("\n\n");
 }
 
-void print_sentence(FILE *fp, Sentence sent, int w) {
-/* this version just prints it on one line.  */
+/**
+ * this version just prints it on one line. 
+ */
+void print_sentence(FILE *fp, Sentence sent, int w)
+{
     int i;
     if (sent->dict->left_wall_defined) i=1; else i=0;
     for (; i<sent->length - sent->dict->right_wall_defined; i++) {
-	fprintf(fp, "%s ", sent->word[i].string);
+        fprintf(fp, "%s ", sent->word[i].string);
     }
     fprintf(fp, "\n");
 }
 
-char * trailer(int mode) {
-    static char * trailer_string=
+const char * trailer(int mode)
+{
+    static const char * trailer_string=
         "diagram\n"
-	"\n"
-	"%%EndDocument\n"
-	;
+        "\n"
+        "%%EndDocument\n"
+        ;
 
     if (mode==1) return trailer_string;
     else return "";
 }
 
-char * header(int mode) {
-    static char * header_string=
+const char * header(int mode)
+{
+    static const char * header_string=
         "%!PS-Adobe-2.0 EPSF-1.2\n"
         "%%Pages: 1\n"
         "%%BoundingBox: 0 -20 500 200\n"
