@@ -41,21 +41,25 @@
 
    */
 
-int hash_string(unsigned char *str, String_set *ss) {
+static int hash_string(const char *sa, const String_set *ss)
+{
+    unsigned char *str = (unsigned char *) sa;
     unsigned int accum = 0;
     for (;*str != '\0'; str++) accum = ((256*accum) + (*str)) % (ss->size);
     return accum;
 }
 
-int stride_hash_string(unsigned char *str, String_set *ss) {
-  /* This is the stride used, so we have to make sure that its value is not 0 */
+static int stride_hash_string(const char *sa, const String_set *ss)
+{
+    unsigned char *str = (unsigned char *) sa;
+    /* This is the stride used, so we have to make sure that its value is not 0 */
     unsigned int accum = 0;
     for (;*str != '\0'; str++) accum = ((17*accum) + (*str)) % (ss->size);
     if (accum == 0) accum = 1;
     return accum;
 }
 
-int next_prime_up(int start) {
+static int next_prime_up(int start) {
 /* return the next prime up from start */
     int i;
     start = start | 1; /* make it odd */
@@ -82,7 +86,7 @@ String_set * string_set_create(void) {
     return ss;
 }
 
-int find_place(char * str, String_set *ss) {
+static int find_place(const char * str, String_set *ss) {
     /* lookup the given string in the table.  Return a pointer
        to the place it is, or the place where it should be. */
     int h, s, i;
@@ -93,7 +97,7 @@ int find_place(char * str, String_set *ss) {
     }
 }
 
-void grow_table(String_set *ss) {
+static void grow_table(String_set *ss) {
     String_set old;
     int i, p;
     
@@ -114,7 +118,7 @@ void grow_table(String_set *ss) {
     xfree((char *) old.table, old.size * sizeof(char *));
 }
 
-char * string_set_add(char * source_string, String_set * ss) {
+char * string_set_add(const char * source_string, String_set * ss) {
     char * str;
     int len, p;
     
@@ -137,7 +141,7 @@ char * string_set_add(char * source_string, String_set * ss) {
     return str;
 }
 
-char * string_set_lookup(char * source_string, String_set * ss) {
+char * string_set_lookup(const char * source_string, String_set * ss) {
     int p;
     
     p = find_place(source_string, ss);
