@@ -1,15 +1,15 @@
-/********************************************************************************/
-/* Copyright (c) 2004                                                           */
-/* Daniel Sleator, David Temperley, and John Lafferty                           */
-/* All rights reserved                                                          */
-/*                                                                              */
-/* Use of the link grammar parsing system is subject to the terms of the        */
-/* license set forth in the LICENSE file included with this software,           */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html           */
-/* This license allows free redistribution and use in source and binary         */
-/* forms, with or without modification, subject to certain conditions.          */
-/*                                                                              */
-/********************************************************************************/
+/*************************************************************************/
+/* Copyright (c) 2004                                                    */
+/* Daniel Sleator, David Temperley, and John Lafferty                    */
+/* All rights reserved                                                   */
+/*                                                                       */
+/* Use of the link grammar parsing system is subject to the terms of the */
+/* license set forth in the LICENSE file included with this software,    */
+/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* This license allows free redistribution and use in source and binary  */
+/* forms, with or without modification, subject to certain conditions.   */
+/*                                                                       */
+/*************************************************************************/
 
 #ifndef _STRUCTURESH_
 #define _STRUCTURESH_
@@ -185,6 +185,9 @@ OBS #define AMPERSAND_WORD   ("AMPERSAND")
 #define SHORT_LEN 6
 #define NO_WORD 255
 
+#define PARSE_NUM_OVERFLOW (1LL<<24)  
+typedef long long s64; /* signed 64-bit integer, even on 32-bit cpus */
+
 typedef struct Connector_struct Connector;
 struct Connector_struct
 {
@@ -218,10 +221,11 @@ struct Disjunct_struct
 };
 
 typedef struct Link_s * Link;
-struct Link_s {
+struct Link_s
+{
     int l, r;
     Connector * lc, * rc;
-    char * name;              /* spelling of full link name */
+    const char * name;              /* spelling of full link name */
 };
 
 typedef struct Match_node_struct Match_node;
@@ -232,8 +236,9 @@ struct Match_node_struct {
 
 typedef struct Exp_struct Exp;
 typedef struct X_node_struct X_node;
-struct X_node_struct {
-    char * string;  /* the word itself */
+struct X_node_struct
+{
+    const char * string;  /* the word itself */
     Exp * exp;
     X_node *next;
 };
@@ -290,8 +295,9 @@ struct Word_file_struct {
 /* the result of a dictionary lookup.                                     */
 
 typedef struct Dict_node_struct Dict_node;
-struct Dict_node_struct {
-    char      * string;  /* the word itself */
+struct Dict_node_struct
+{
+    const char * string;  /* the word itself */
     Word_file * file;    /* the file the word came from (NULL if dict file) */
     Exp       * exp;
     Dict_node *left, *right;
@@ -372,7 +378,7 @@ struct Parse_choice_struct {
     Disjunct *ld, *md, *rd;  /* the chosen disjuncts for the relevant three words */
 };
 struct Parse_set_struct {
-    int count;  /* the number of ways */
+    s64 count;  /* the number of ways */
     Parse_choice * first;
     Parse_choice * current;  /* used to enumerate linkages */
 };
@@ -397,7 +403,7 @@ typedef struct {
 /* from pp_linkset.c */
 typedef struct pp_linkset_node_s
 {
-    char *str;
+    const char *str;
     struct pp_linkset_node_s *next;
 } pp_linkset_node;
 
