@@ -1,15 +1,15 @@
-/********************************************************************************/
-/* Copyright (c) 2004                                                           */
-/* Daniel Sleator, David Temperley, and John Lafferty                           */
-/* All rights reserved                                                          */
-/*                                                                              */
-/* Use of the link grammar parsing system is subject to the terms of the        */
-/* license set forth in the LICENSE file included with this software,           */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html           */
-/* This license allows free redistribution and use in source and binary         */
-/* forms, with or without modification, subject to certain conditions.          */
-/*                                                                              */
-/********************************************************************************/
+/*************************************************************************/
+/* Copyright (c) 2004                                                    */
+/* Daniel Sleator, David Temperley, and John Lafferty                    */
+/* All rights reserved                                                   */
+/*                                                                       */
+/* Use of the link grammar parsing system is subject to the terms of the */
+/* license set forth in the LICENSE file included with this software,    */
+/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* This license allows free redistribution and use in source and binary  */
+/* forms, with or without modification, subject to certain conditions.   */
+/*                                                                       */
+/*************************************************************************/
 
 #include <link-grammar/api.h>
 
@@ -19,56 +19,88 @@ int post_quote[MAX_SENTENCE];
 /*static char * strip_left[] = {"(", "$", "``", NULL}; */
 /*static char * strip_right[] = {")", "%", ",", ".", ":", ";", "?", "!", "''", "'", "'s", NULL};*/
 
-static int is_initials_word(const char * word) {
-	/* This is rather esoteric and not terribly important.
-	   It returns TRUE if the word matches the pattern /[A-Z]\.]+/
-	   */
-	int i;
-	for (i=0; word[i] != '\0'; i += 2) {
-		if (!isupper((int)word[i])) return FALSE;
-		if (word[i+1] != '.') return FALSE;
+/**
+ * This is rather esoteric and not terribly important.
+ * It returns TRUE if the word matches the pattern /[A-Z]\.]+/
+ */
+static int is_initials_word(const char * word)
+{
+	int i=0;
+	while (word[i])
+	{
+		int nb = is_utf8_upper(&word[i]);
+		if (!nb) return FALSE;
+		i += nb;
+		if (word[i] != '.') return FALSE;
+		i++;
 	}
 	return TRUE;
 }
 
-static int is_number(const char * s) {
-	if (!isdigit((int)*s)) return FALSE;
-	for (; *s != '\0'; s++) {
-		if (isdigit((int)*s) || (*s == '.') || (*s == ',') || (*s == ':')) continue;
-		/* The ":" is included here so we allow "10:30" to be a number. */
-		return FALSE;
+/**
+ * The ":" is included here so we allow "10:30" to be a number.
+ * We also allow U+00A0 "no-break space"
+ */
+static int is_number(const char * s)
+{
+	wchar_t c;
+	if (!is_utf8_digit(s)) return FALSE;
+
+	while (*s != 0)
+	{
+		int nb = mbtowc(&c, s, 4);
+		if (iswdigit(c)) { s += nb; }
+
+		/* U+00A0 no break space */
+		else if (0xa0 == c) { s += nb; }
+
+		else if ((*s == '.') || (*s == ',') || (*s == ':')) { s++; }
+		else return FALSE;
 	}
 	return TRUE;
 }
 
-static int ishyphenated(const char * s) {
-/* returns TRUE iff it's an appropriately formed hyphenated word.
-   This means all letters, numbers, or hyphens, not beginning and
-   ending with a hyphen.
-*/
+/**
+ * Returns TRUE iff it's an appropriately formed hyphenated word.
+ * This means all letters, numbers, or hyphens, not beginning and
+ * ending with a hyphen.
+ */
+static int ishyphenated(const char * s)
+{
+	wchar_t c;
 	int hyp, nonalpha;
 	hyp = nonalpha = 0;
+
 	if (*s == '-') return FALSE;
-	while (*s != '\0') {
-		if (!isalpha((int)*s) && !isdigit((int)*s) && (*s!='.') && (*s!=',')
+
+	while (*s != '\0')
+	{
+		int nb = mbtowc(&c, s, 4);
+
+		if (!iswalpha(c) && !iswdigit(c) && (*s!='.') && (*s!=',')
 			&& (*s!='-')) return FALSE;
 		if (*s == '-') hyp++;
-		s++;
+		s += nb;
 	}
 	return ((*(s-1)!='-') && (hyp>0));
 }
 
-static int is_ing_word(const char * s) {
-
+/** 
+ * The following four routines implement a cheap-hack morphology for
+ * English.
+ * XXX this is wrong for non-english
+ */
+static int is_ing_word(const char * s)
+{
 	int i=0;
 	for (; *s != '\0'; s++) i++;
-	if(i<5) return FALSE;
-	if(strncmp("ing", s-3, 3)==0) return TRUE;
+	if (i<5) return FALSE;
+	if (strncmp("ing", s-3, 3)==0) return TRUE;
 	return FALSE;
 }
 
-static int is_s_word(const char * s) {
-
+static int is_s_word(const char * s)
+{
 	for (; *s != '\0'; s++);
 	s--;
 	if(*s != 's') return FALSE;
@@ -77,27 +109,31 @@ static int is_s_word(const char * s) {
 	return TRUE;
 }
 
-static int is_ed_word(const char * s) {
-
+static int is_ed_word(const char * s)
+{
 	int i=0;
 	for (; *s != '\0'; s++) i++;
-	if(i<4) return FALSE;
-	if(strncmp("ed", s-2, 2)==0) return TRUE;
+	if (i<4) return FALSE;
+	if (strncmp("ed", s-2, 2)==0) return TRUE;
 	return FALSE;
 }
 
-static int is_ly_word(const char * s) {
-
+static int is_ly_word(const char * s)
+{
 	int i=0;
 	for (; *s != '\0'; s++) i++;
-	if(i<4) return FALSE;
-	if(strncmp("ly", s-2, 2)==0) return TRUE;
+	if (i<4) return FALSE;
+	if (strncmp("ly", s-2, 2)==0) return TRUE;
 	return FALSE;
 }
 
-static int issue_sentence_word(Sentence sent, const char * s) {
-	/* the string s is the next word of the sentence do not issue the empty
-	 * string.  return false if too many words or the word is too long.  */
+/** 
+ * The string s is the next word of the sentence. 
+ * Do not issue the empty string.  
+ * Return false if too many words or the word is too long. 
+ */
+static int issue_sentence_word(Sentence sent, const char * s)
+{
 	if (*s == '\0') return TRUE;
 	if (strlen(s) > MAX_WORD) {
 		lperror(SEPARATE,
@@ -110,11 +146,13 @@ static int issue_sentence_word(Sentence sent, const char * s) {
 		lperror(SEPARATE, ". The sentence has too many words.\n");
 		return FALSE;
 	}
+
 	strcpy(sent->word[sent->length].string, s);
+
 	/* Now we record whether the first character of the word is upper-case.
 	   (The first character may be made lower-case
 	   later, but we may want to get at the original version) */
-	if(isupper((int)s[0])) sent->word[sent->length].firstupper=1;
+	if (is_utf8_upper(s)) sent->word[sent->length].firstupper=1;
 	else sent->word[sent->length].firstupper = 0;
 	sent->length++;
 	return TRUE;
@@ -175,20 +213,52 @@ static int issue_sentence_word(Sentence sent, const char * s) {
 #undef		MIN
 #define MIN(a, b)  (((a) < (b)) ? (a) : (b))
 
-static int separate_word(Sentence sent, char *w, char *wend, int is_first_word, int quote_found) {
+static int downcase_is_in_dict(Dictionary dict, char * word)
+{
+	int i, rc;
+	char low[MB_CUR_MAX];
+	char save[MB_CUR_MAX];
+	wchar_t c;
+	int nbl, nbh;
+
+	if (!is_utf8_upper(word)) return FALSE;
+
+	nbh = mbtowc (&c, word, 4);
+	c = towlower(c);
+	nbl = wctomb(low, c);
+	if (nbh != nbl)
+	{
+		fprintf(stderr, "Error: can't downcase multi-byte string!\n");
+		return FALSE;
+	}
+
+	/* Downcase */
+	for (i=0; i<nbl; i++) { save[i] = word[i]; word[i] = low[i]; }
+
+	/* Look it up, then restore old value */
+	rc = boolean_dictionary_lookup(dict, word);
+	for (i=0; i<nbh; i++) { word[i] = save[i]; }
+
+	return rc; 
+}
+
+static int separate_word(Sentence sent, char *w, char *wend, int is_first_word, int quote_found)
+{
 	/* w points to a string, wend points to the char one after the end.  The
 	 * "word" w contains no blanks.  This function splits up the word if
 	 * necessary, and calls "issue_sentence_word()" on each of the resulting
 	 * parts.  The process is described above.  returns TRUE of OK, FALSE if
 	 * too many punctuation marks */
 	int i, j, k, l, len;
-	int r_strippable=0, l_strippable=0, s_strippable=0, p_strippable=0, n_r_stripped, s_stripped,
-	  word_is_in_dict, s_ok;
+	int r_strippable=0, l_strippable=0;
+	int s_strippable=0, p_strippable=0;
+	int  n_r_stripped, s_stripped;
+	int word_is_in_dict, s_ok;
 	int r_stripped[MAX_STRIP];  /* these were stripped from the right */
-	char ** strip_left=NULL;
-	char ** strip_right=NULL;
-	char ** prefix=NULL;
-	char ** suffix=NULL;
+	const char ** strip_left=NULL;
+	const char ** strip_right=NULL;
+	const char ** prefix=NULL;
+	const char ** suffix=NULL;
 	char word[MAX_WORD+1];
 	char newword[MAX_WORD+1];
 	Dict_node * dn, * dn2, * start_dn;
@@ -197,46 +267,48 @@ static int separate_word(Sentence sent, char *w, char *wend, int is_first_word,
 	const char * suf_con = "SUF";
 	const char * pre_con = "PRE";
 
-	if(sent->dict->affix_table!=NULL) {
+	if (sent->dict->affix_table!=NULL)
+	{
+		start_dn = list_whole_dictionary(sent->dict->affix_table->root, NULL);
+		for (dn = start_dn; dn != NULL; dn = dn->right)
+		{
+			if (word_has_connector(dn, rpunc_con, 0)) r_strippable++;
+			if (word_has_connector(dn, lpunc_con, 0)) l_strippable++;
+			if (word_has_connector(dn, suf_con, 0)) s_strippable++;
+			if (word_has_connector(dn, pre_con, 0)) p_strippable++;
+	  	}
+		strip_right = (const char **) xalloc(r_strippable * sizeof(char *));
+		strip_left = (const char **) xalloc(l_strippable * sizeof(char *));
+		suffix = (const char **) xalloc(s_strippable * sizeof(char *));
+		prefix = (const char **) xalloc(p_strippable * sizeof(char *));
 
-	  start_dn = list_whole_dictionary(sent->dict->affix_table->root, NULL);
-	  for(dn=start_dn; dn!=NULL; dn=dn->right) {
-		if(word_has_connector(dn, rpunc_con, 0)) r_strippable++;
-		if(word_has_connector(dn, lpunc_con, 0)) l_strippable++;
-		if(word_has_connector(dn, suf_con, 0)) s_strippable++;
-		if(word_has_connector(dn, pre_con, 0)) p_strippable++;
-	  }
-	  strip_right = (char **) xalloc(r_strippable * sizeof(char *));
-	  strip_left = (char **) xalloc(l_strippable * sizeof(char *));
-	  suffix = (char **) xalloc(s_strippable * sizeof(char *));
-	  prefix = (char **) xalloc(p_strippable * sizeof(char *));
-
-	  i=0;
-	  j=0;
-	  k=0;
-	  l=0;
-	  dn=start_dn;
-	  while(dn!=NULL) {
-		if(word_has_connector(dn, rpunc_con, 0)) {
-		  strip_right[i] = dn->string;
-		  i++;
+		i=0;
+		j=0;
+		k=0;
+		l=0;
+		dn = start_dn;
+		while (dn != NULL)
+		{
+			if(word_has_connector(dn, rpunc_con, 0)) {
+				strip_right[i] = dn->string;
+				i++;
+			}
+			if(word_has_connector(dn, lpunc_con, 0)) {
+				strip_left[j] = dn->string;
+				j++;
+			}
+			if(word_has_connector(dn, suf_con, 0)) {
+				suffix[k] = dn->string;
+				k++;
+			}
+			if(word_has_connector(dn, pre_con, 0)) {
+				prefix[l] = dn->string;
+				l++;
+			}
+			dn2=dn->right;
+			xfree(dn, sizeof(Dict_node));
+			dn=dn2;
 		}
-		if(word_has_connector(dn, lpunc_con, 0)) {
-		  strip_left[j] = dn->string;
-		  j++;
-		}
-		if(word_has_connector(dn, suf_con, 0)) {
-		  suffix[k] = dn->string;
-		  k++;
-		}
-		if(word_has_connector(dn, pre_con, 0)) {
-		  prefix[l] = dn->string;
-		  l++;
-		}
-		dn2=dn->right;
-		xfree(dn, sizeof(Dict_node));
-		dn=dn2;
-	  }
 	}
 
 	for (;;) {
@@ -250,80 +322,90 @@ static int separate_word(Sentence sent, char *w, char *wend, int is_first_word,
 		if (i==l_strippable) break;
 	}
 
-	/* Now w points to the string starting just to the right of any left-stripped characters. */
-	/* stripped[] is an array of numbers, indicating the index numbers (in the strip_right array) of any
-	   strings stripped off; stripped[0] is the number of the first string stripped off, etc. When it
-	   breaks out of this loop, n_stripped will be the number of strings stripped off. */
-
-	for (n_r_stripped = 0; n_r_stripped < MAX_STRIP; n_r_stripped++) {
-
+	/* Now w points to the string starting just to the right of
+	 * any left-stripped characters.
+	 * stripped[] is an array of numbers, indicating the index
+	 * numbers (in the strip_right array) of any strings stripped off;
+	 * stripped[0] is the number of the first string stripped off, etc.
+	 * When it breaks out of this loop, n_stripped will be the number
+	 * of strings stripped off.
+	 */
+	for (n_r_stripped = 0; n_r_stripped < MAX_STRIP; n_r_stripped++) 
+	{
 		strncpy(word, w, MIN(wend-w, MAX_WORD));
 		word[MIN(wend-w, MAX_WORD)] = '\0';
 		if (wend == w) break;  /* it will work without this */
 
 		if (boolean_dictionary_lookup(sent->dict, word) || is_initials_word(word)) break;
-		if (is_first_word && isupper((int)word[0])) {
-		  /* This should happen if it's a word after a colon, also! */
-			word[0] = tolower(word[0]);
-			if (boolean_dictionary_lookup(sent->dict, word)) {
-				word[0] = toupper(word[0]);  /* restore word to what it was */
-				break;
-			}
-			word[0] = toupper(word[0]);
-		}
-		for (i=0; i < r_strippable; i++) {
+
+		/* This could happen if it's a word after a colon, also! */
+		if (is_first_word && downcase_is_in_dict (sent->dict, word)) break;
+
+		for (i=0; i < r_strippable; i++)
+		{
 			len = strlen(strip_right[i]);
-			if ((wend-w) < len) continue;  /* the remaining w is too short for a possible match */
+
+			/* the remaining w is too short for a possible match */
+			if ((wend-w) < len) continue;
 			if (strncmp(wend-len, strip_right[i], len) == 0) {
 				r_stripped[n_r_stripped] = i;
 				wend -= len;
 				break;
 			}
 		}
-		if(i==r_strippable) break;
+		if (i == r_strippable) break;
 	}
 
-	/* Now we strip off suffixes...w points to the remaining word, "wend" to the end of the word. */
+	/* Now we strip off suffixes...w points to the remaining word, 
+	 * "wend" to the end of the word. */
 
 	s_stripped = -1;
 	strncpy(word, w, MIN(wend-w, MAX_WORD));
 	word[MIN(wend-w, MAX_WORD)] = '\0';
 	word_is_in_dict=0;
 
-	if (boolean_dictionary_lookup(sent->dict, word) || is_initials_word(word)) word_is_in_dict=1;
-	if (is_first_word && isupper((int)word[0])) {
-	  word[0] = tolower(word[0]);
-	  if (boolean_dictionary_lookup(sent->dict, word)) word_is_in_dict=1;
-	  word[0] = toupper(word[0]);
-	}
-	if(word_is_in_dict==0) {
+	if (boolean_dictionary_lookup(sent->dict, word))
+		word_is_in_dict = 1;
+	else if (is_initials_word(word))
+		word_is_in_dict = 1;
+	else if (is_first_word && downcase_is_in_dict (sent->dict,word))
+		word_is_in_dict = 1;
+
+	if(word_is_in_dict==0)
+	{
 	  j=0;
-	  for (i=0; i <= s_strippable; i++) {
+	  for (i=0; i < s_strippable+1; i++) {
 		s_ok = 0;
-		/* Go through once for each suffix; then go through one final time for the no-suffix case */
+		/* Go through once for each suffix; then go through one 
+		 * final time for the no-suffix case */
 		if(i < s_strippable) {
 		  len = strlen(suffix[i]);
-		  if ((wend-w) < len) continue;  /* the remaining w is too short for a possible match */
+
+		  /* the remaining w is too short for a possible match */
+		  if ((wend-w) < len) continue;
+
 		  if (strncmp(wend-len, suffix[i], len) == 0) s_ok=1;
 				  }
 		else len=0;
 
-		if(s_ok==1 || i==s_strippable) {
+		if(s_ok==1 || i==s_strippable)
+		{
+			strncpy(newword, w, MIN((wend-len)-w, MAX_WORD));
+			newword[MIN((wend-len)-w, MAX_WORD)] = '\0';
 
-		  strncpy(newword, w, MIN((wend-len)-w, MAX_WORD));
-		  newword[MIN((wend-len)-w, MAX_WORD)] = '\0';
+			/* Check if the remainder is in the dictionary;
+			 * for the no-suffix case, it won't be */
+			if (boolean_dictionary_lookup(sent->dict, newword)) {
+				if(verbosity>1) if(i< s_strippable) printf("Splitting word into two: %s-%s\n", newword, suffix[i]);
+				s_stripped = i;
+				wend -= len;
+				strncpy(word, w, MIN(wend-w, MAX_WORD));
+				word[MIN(wend-w, MAX_WORD)] = '\0';
+				break;
+			}
 
-		  /* Check if the remainder is in the dictionary; for the no-suffix case, it won't be */
-		  if (boolean_dictionary_lookup(sent->dict, newword)) {
-			if(verbosity>1) if(i< s_strippable) printf("Splitting word into two: %s-%s\n", newword, suffix[i]);
-			s_stripped = i;
-			wend -= len;
-			strncpy(word, w, MIN(wend-w, MAX_WORD));
-			word[MIN(wend-w, MAX_WORD)] = '\0';
-			break;
-		  }
-
-		  /* If the remainder isn't in the dictionary, try stripping off prefixes */
+			/* If the remainder isn't in the dictionary, 
+			 * try stripping off prefixes */
 		  else {
 			for (j=0; j<p_strippable; j++) {
 			  if (strncmp(w, prefix[j], strlen(prefix[j])) == 0) {
@@ -393,12 +475,14 @@ static int separate_word(Sentence sent, char *w, char *wend, int is_first_word,
 	return TRUE;
 }
 
-int separate_sentence(char * s, Sentence sent) {
-	/* The string s has just been read in from standard input.
-	   This function breaks it up into words and stores these words in
-	   the sent->word[] array.  Returns TRUE if all is well, FALSE otherwise.
-	   Quote marks are treated just like blanks.
-	   */
+/**
+ * The string s has just been read in from standard input.
+ * This function breaks it up into words and stores these words in
+ * the sent->word[] array.  Returns TRUE if all is well, FALSE otherwise.
+ * Quote marks are treated just like blanks.
+ */
+int separate_sentence(char * s, Sentence sent)
+{
 	char *t;
 	int i, is_first, quote_found;
 	Dictionary dict = sent->dict;
@@ -410,14 +494,30 @@ int separate_sentence(char * s, Sentence sent) {
 		if (!issue_sentence_word(sent, LEFT_WALL_WORD)) return FALSE;
 
 	is_first = TRUE;
-	for(;;) {
+	for(;;) 
+	{
+		wchar_t c;
+		int nb = mbtowc(&c, s, 4);
 		quote_found = FALSE;
-		while(isspace((int)*s) || (*s == '\"')) {
-		  s++;
-		  if(*s == '\"') quote_found=TRUE;
+
+		/* skip all whitespace */
+		while (iswspace(c) || (c == '\"'))
+		{
+			s += nb;
+			if(*s == '\"') quote_found=TRUE;
+			nb = mbtowc(&c, s, 4);
 		}
+
 		if (*s == '\0') break;
-		for (t=s; !((isspace((int)*t) || (*t == '\"')) || *t=='\0'); t++);
+
+		t = s;
+		nb = mbtowc(&c, t, 4);
+		while (!iswspace(c) && (c != '\"') && (c != 0))
+		{
+			t += nb;
+			nb = mbtowc(&c, t, 4);
+		}
+
 		if (!separate_word(sent, s, t, is_first, quote_found)) return FALSE;
 		is_first = FALSE;
 		s = t;
@@ -525,29 +625,30 @@ static void handle_unknown_word(Sentence sent, int i, char * s) {
 	}
 }
 
-int build_sentence_expressions(Sentence sent) {
-	/* Corrects case of first word, fills in other proper nouns, and
-	   builds the expression lists for the resulting words.
-
-	   Algorithm:
-	   Apply the following step to all words w:
-	   if w is in the dictionary, use it.
-	   else if w is upper case use PROPER_WORD disjuncts for w.
-	   else if it's hyphenated, use HYPHENATED_WORD
-	   else if it's a number, use NUMBER_WORD.
-
-	   Now, we correct the first word, w.
-	   if w is upper case, let w' be the lower case version of w.
-	   if both w and w' are in the dict, concatenate these disjncts.
-	   else if w' is in dict, use disjuncts of w'
-	   else leave the disjuncts alone
-	   */
+/**
+ * Corrects case of first word, fills in other proper nouns, and
+ * builds the expression lists for the resulting words.
+ *
+ * Algorithm:
+ * Apply the following step to all words w:
+ * if w is in the dictionary, use it.
+ * else if w is upper case use PROPER_WORD disjuncts for w.
+ * else if it's hyphenated, use HYPHENATED_WORD
+ * else if it's a number, use NUMBER_WORD.
+ *
+ * Now, we correct the first word, w.
+ * if w is upper case, let w' be the lower case version of w.
+ * if both w and w' are in the dict, concatenate these disjncts.
+ * else if w' is in dict, use disjuncts of w'
+ * else leave the disjuncts alone
+ */
+int build_sentence_expressions(Sentence sent)
+{
 	int i, first_word;  /* the index of the first word after the wall */
 	char *s, *u, temp_word[MAX_WORD+1];
 	X_node * e;
 	Dictionary dict = sent->dict;
 
-
 	if (dict->left_wall_defined) {
 		first_word = 1;
 	} else {
@@ -556,13 +657,14 @@ int build_sentence_expressions(Sentence sent) {
 
 	/* the following loop treats all words the same
 	   (nothing special for 1st word) */
-	for (i=0; i<sent->length; i++) {
+	for (i=0; i<sent->length; i++)
+	{
 		s = sent->word[i].string;
 		if (boolean_dictionary_lookup(sent->dict, s)) {
 			sent->word[i].x = build_word_expressions(sent, s);
-		} else if (isupper((int)s[0]) && is_s_word(s) && dict->pl_capitalized_word_defined) {
+		} else if (is_utf8_upper(s) && is_s_word(s) && dict->pl_capitalized_word_defined) {
 			if (!special_string(sent, i, PL_PROPER_WORD)) return FALSE;
-		} else if (isupper((int)s[0]) && dict->capitalized_word_defined) {
+		} else if (is_utf8_upper(s) && dict->capitalized_word_defined) {
 			if (!special_string(sent, i, PROPER_WORD)) return FALSE;
 		} else if (is_number(s) && dict->number_word_defined) {
 			/* we know it's a plural number, or 1 */
@@ -588,30 +690,39 @@ int build_sentence_expressions(Sentence sent) {
 		}
 	}
 
-	/* Under certain cases--if it's the first word of the sentence, or if it follows a colon
-	   or a quotation mark--a word that's capitalized has to be looked up as an uncapitalized
-	   word (as well as a capitalized word). */
-
-
+	/* Under certain cases--if it's the first word of the sentence,
+	 * or if it follows a colon or a quotation mark--a word that's 
+	 * capitalized has to be looked up as an uncapitalized word
+	 * (as well as a capitalized word).
+	 */
 	for (i=0; i<sent->length; i++) {
 		if (! (i==first_word || (i>0 && strcmp(":", sent->word[i-1].string)==0) || post_quote[i]==1) ) continue;
 		s = sent->word[i].string;
-		if (isupper((int)s[0])) {
-			safe_strcpy(temp_word, s, sizeof(temp_word));
-			temp_word[0] = tolower(temp_word[0]);
+
+		if (is_utf8_upper(s))
+		{
+			downcase_utf8_str(temp_word, s, MAX_WORD);
 			u = string_set_add(temp_word, sent->string_set);
+
 			/* If the lower-case version is in the dictionary... */
-			if (boolean_dictionary_lookup(sent->dict, u)) {
-				/* Then check if the upper-case version is there. If it is, the disjuncts for
-				   the upper-case version have been put there already. So add on the disjuncts
-				   for the lower-case version. */
-				if (boolean_dictionary_lookup(sent->dict, s)) {
+			if (boolean_dictionary_lookup(sent->dict, u))
+			{
+				/* Then check if the upper-case version is there. 
+				 * If it is, the disjuncts for the upper-case version 
+				 * have been put there already. So add on the disjuncts
+				 * for the lower-case version. */
+				if (boolean_dictionary_lookup(sent->dict, s))
+				{
 					e = build_word_expressions(sent, u);
 					sent->word[i].x =
 						catenate_X_nodes(sent->word[i].x, e);
-				} else {
-				  /* If the upper-case version isn't there, replace the u.c. disjuncts with l.c. ones */
-					s[0] = tolower(s[0]);
+				} 
+				else
+				{
+					/* If the upper-case version isn't there,
+					 * replace the u.c. disjuncts with l.c. ones.
+					 */
+					safe_strcpy(s,u, MAX_WORD);
 					e = build_word_expressions(sent, s);
 					free_X_nodes(sent->word[i].x);
 					sent->word[i].x = e;
@@ -624,29 +735,33 @@ int build_sentence_expressions(Sentence sent) {
 }
 
 
-/* This just looks up all the words in the sentence, and builds
-   up an appropriate error message in case some are not there.
-   It has no side effect on the sentence.  Returns TRUE if all
-   went well.
+/**
+ * This just looks up all the words in the sentence, and builds
+ * up an appropriate error message in case some are not there.
+ * It has no side effect on the sentence.  Returns TRUE if all
+ * went well.
  */
-int sentence_in_dictionary(Sentence sent){
+int sentence_in_dictionary(Sentence sent)
+{
 	int w, ok_so_far;
 	char * s;
 	Dictionary dict = sent->dict;
 	char temp[1024];
 
 	ok_so_far = TRUE;
-	for (w=0; w<sent->length; w++) {
+	for (w=0; w<sent->length; w++)
+	{
 		s = sent->word[w].string;
 		if (!boolean_dictionary_lookup(dict, s) &&
-			!(isupper((int)s[0])   && dict->capitalized_word_defined) &&
-			!(isupper((int)s[0]) && is_s_word(s) && dict->pl_capitalized_word_defined) &&
-			!(ishyphenated(s) && dict->hyphenated_word_defined)  &&
-			!(is_number(s)	&& dict->number_word_defined) &&
-			!(is_ing_word(s)  && dict->ing_word_defined)  &&
-			!(is_s_word(s)	&& dict->s_word_defined)  &&
-			!(is_ed_word(s)   && dict->ed_word_defined)  &&
-			!(is_ly_word(s)   && dict->ly_word_defined)) {
+		    !(is_utf8_upper(s)   && dict->capitalized_word_defined) &&
+		    !(is_utf8_upper(s) && is_s_word(s) && dict->pl_capitalized_word_defined) &&
+		    !(ishyphenated(s) && dict->hyphenated_word_defined)  &&
+		    !(is_number(s)	&& dict->number_word_defined) &&
+		    !(is_ing_word(s)  && dict->ing_word_defined)  &&
+		    !(is_s_word(s)	&& dict->s_word_defined)  &&
+		    !(is_ed_word(s)   && dict->ed_word_defined)  &&
+		    !(is_ly_word(s)   && dict->ly_word_defined))
+		{
 			if (ok_so_far) {
 				safe_strcpy(temp, "The following words are not in the dictionary:", sizeof(temp));
 				ok_so_far = FALSE;
