@@ -1,15 +1,15 @@
-/********************************************************************************/
-/* Copyright (c) 2004                                                           */
-/* Daniel Sleator, David Temperley, and John Lafferty                           */
-/* All rights reserved                                                          */
-/*                                                                              */
-/* Use of the link grammar parsing system is subject to the terms of the        */
-/* license set forth in the LICENSE file included with this software,           */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html           */
-/* This license allows free redistribution and use in source and binary         */
-/* forms, with or without modification, subject to certain conditions.          */
-/*                                                                              */
-/********************************************************************************/
+/*************************************************************************/
+/* Copyright (c) 2004                                                    */
+/* Daniel Sleator, David Temperley, and John Lafferty                    */
+/* All rights reserved                                                   */
+/*                                                                       */
+/* Use of the link grammar parsing system is subject to the terms of the */
+/* license set forth in the LICENSE file included with this software,    */
+/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* This license allows free redistribution and use in source and binary  */
+/* forms, with or without modification, subject to certain conditions.   */
+/*                                                                       */
+/*************************************************************************/
 
 #include <link-grammar/api.h>
 #include <locale.h>
@@ -44,20 +44,89 @@ char *safe_strdup(const char *u)
 	return NULL;
 }
 
-void safe_strcpy(char *u, const char * v, int usize) {
-/* Copies as much of v into u as it can assuming u is of size usize */
-/* guaranteed to terminate u with a '\0'.						   */
+/**
+ * Copies as much of v into u as it can assuming u is of size usize
+ * guaranteed to terminate u with a '\0'.
+ */
+void safe_strcpy(char *u, const char * v, size_t usize)
+{
 	strncpy(u, v, usize-1);
 	u[usize-1] = '\0';
 }
 
-void safe_strcat(char *u, const char *v, int usize) {
-/* catenates as much of v onto u as it can assuming u is of size usize		   */
-/* guaranteed to terminate u with a '\0'.  Assumes u and v are null terminated.  */
+/**
+ * Catenates as much of v onto u as it can assuming u is of size usize
+ * guaranteed to terminate u with a '\0'.  Assumes u and v are null
+ * terminated.
+ */
+void safe_strcat(char *u, const char *v, size_t usize)
+{
 	strncat(u, v, usize-strlen(u)-1);
 	u[usize-1] = '\0';
 }
 
+/**
+ * Downcase the first letter of the word.
+ */
+void downcase_utf8_str(char *to, const char * from, size_t usize)
+{
+	wchar_t c;
+	int i, nbl, nbh;
+	char low[MB_CUR_MAX];
+
+	nbh = mbtowc (&c, from, 4);
+	c = towlower(c);
+	nbl = wctomb(low, c);
+
+	/* Check for error on an in-place copy */
+	if ((nbh < nbl) && (to == from))
+	{
+		/* I'm to lazy to fix this */
+		fprintf(stderr, "Error: can't downcase multi-byte string!\n");
+		return;
+	}
+
+	/* Downcase */
+	for (i=0; i<nbl; i++) { to[i] = low[i]; }
+
+	if ((nbh == nbl) && (to == from)) return;
+
+	from += nbh;
+	to += nbl;
+	safe_strcpy(to, from, usize-nbl);
+}
+
+/**
+ * Upcase the first letter of the word.
+ */
+void upcase_utf8_str(char *to, const char * from, size_t usize)
+{
+	wchar_t c;
+	int i, nbl, nbh;
+	char low[MB_CUR_MAX];
+
+	nbh = mbtowc (&c, from, 4);
+	c = towupper(c);
+	nbl = wctomb(low, c);
+
+	/* Check for error on an in-place copy */
+	if ((nbh < nbl) && (to == from))
+	{
+		/* I'm to lazy to fix this */
+		fprintf(stderr, "Error: can't upcase multi-byte string!\n");
+		return;
+	}
+
+	/* Upcase */
+	for (i=0; i<nbl; i++) { to[i] = low[i]; }
+
+	if ((nbh == nbl) && (to == from)) return;
+
+	from += nbh;
+	to += nbl;
+	safe_strcpy(to, from, usize-nbl);
+}
+
 unsigned int randtable[RTSIZE];
 
 /* There is a legitimate question of whether having the hash function	*/
@@ -132,16 +201,6 @@ int next_power_of_two_up(int i) {
 	return j;
 }
 
-int upper_case_match(const char *s, const char *t) {
-/* returns TRUE if the initial upper case letters of s and t match */
-	while(isupper((int)*s) || isupper((int)*t)) {
-		if (*s != *t) return FALSE;
-		s++;
-		t++;
-	}
-	return (!isupper((int)*s) && !isupper((int)*t));
-}
-
 void left_print_string(FILE * fp, const char * s, const char * t) {
 /* prints s then prints the last |t|-|s| characters of t.
    if s is longer than t, it truncates s.
@@ -178,7 +237,7 @@ path_get_dirname (const char *file_name)
 	if (!base)
 	{
 #ifdef _WIN32
-		if (isalpha (file_name[0]) && file_name[1] == ':')
+		if (is_utf8_alpha (file_name) && file_name[1] == ':')
 		{
 			char drive_colon_dot[4];
 
@@ -209,7 +268,7 @@ path_get_dirname (const char *file_name)
 	 * (\\server\share\foo), include the slash after the share name,
 	 * returning \\server\share\ .
 	 */
-	if (base == file_name + 1 && isalpha (file_name[0]) && file_name[1] == ':')
+	if (base == file_name + 1 && is_utf8_alpha (file_name) && file_name[1] == ':')
 		base++;
 	else if (IS_DIR_SEPARATOR (file_name[0]) &&
 	         IS_DIR_SEPARATOR (file_name[1]) &&
@@ -250,10 +309,23 @@ path_get_dirname (const char *file_name)
 }
 #endif /* _WIN32 */
 
+char * custom_data_dir = NULL;
+
+void set_data_dir(const char * path)
+{
+	if (custom_data_dir) free (custom_data_dir);
+	custom_data_dir = safe_strdup(path);
+}
+
 static char * get_datadir(void)
 {
 	char * data_dir = NULL;
 
+	if (custom_data_dir != NULL) {
+		data_dir = safe_strdup(custom_data_dir);
+		return data_dir; 
+	}
+	
 #ifdef ENABLE_BINRELOC
 	data_dir = safe_strdup (BR_DATADIR("/link-grammar"));
 #elif defined(_WIN32)
@@ -292,10 +364,23 @@ FILE *dictopen(const char *filename, const char *how)
 {
 	char completename[MAX_PATH_NAME+1];
 	char fulldictpath[MAX_PATH_NAME+1];
+	static char prevpath[MAX_PATH_NAME+1] = "";
+	static int first_time_ever = 1;
 	char *pos, *oldpos;
 	int filenamelen, len;
 	FILE *fp;
 
+	/* Record the first path ever used, so that we can recycle it */
+	if (first_time_ever)
+	{
+		strncpy (prevpath, filename, MAX_PATH_NAME);
+		prevpath[MAX_PATH_NAME] = 0;
+		pos = strrchr(prevpath, '/');
+		if (pos) *pos = 0;
+		pos = strrchr(prevpath, '/');
+		if (pos) *(pos+1) = 0;
+		first_time_ever = 0;
+	}
 	if (filename[0] == '/') {
 		/* fopen returns NULL if the file does not exist. */
 		fp = fopen(filename, how);
@@ -311,20 +396,27 @@ FILE *dictopen(const char *filename, const char *how)
 			free(data_dir);
 		}
 		else {
-			/* always make sure that it ends with a path separator char
-			 * for the below while() loop. */
+			/* Always make sure that it ends with a path
+			 * separator char for the below while() loop. */
 			snprintf(fulldictpath, MAX_PATH_NAME,
-			         "%s%c", DEFAULTPATH, PATH_SEPARATOR);
+			         "%s%c%s%c%s%c%s%c%s%c%s%c",
+				 prevpath, PATH_SEPARATOR,
+				 DEFAULTPATH, PATH_SEPARATOR,
+				 ".", PATH_SEPARATOR,
+				 "data", PATH_SEPARATOR,
+				 "..", PATH_SEPARATOR,
+				 "../data", PATH_SEPARATOR);
 		}
 	}
 
-	/* Now fulldictpath is our dictpath, where each entry is followed by a ":"
-	 * including the last one */
+	/* Now fulldictpath is our dictpath, where each entry is
+	 * followed by a ":" including the last one */
 
 	filenamelen = strlen(filename);
 	len = strlen(fulldictpath)+ filenamelen + 1 + 1;
 	oldpos = fulldictpath;
-	while ((pos = strchr(oldpos, PATH_SEPARATOR)) != NULL) {
+	while ((pos = strchr(oldpos, PATH_SEPARATOR)) != NULL)
+	{
 		strncpy(completename, oldpos, (pos-oldpos));
 		*(completename+(pos-oldpos)) = DIR_SEPARATOR;
 		strcpy(completename+(pos-oldpos)+1,filename);
