@@ -11,27 +11,32 @@
 /*                                                                         */
 /***************************************************************************/
 
+#include <wchar.h>
+#include <wctype.h>
 #include <link-grammar/api.h>
 
-/* Reads in one word from the file, allocates space for it,
- *  and returns it.
+/**
+ * Reads in one word from the file, allocates space for it,
+ * and returns it.
  */
-static char * get_a_word(Dictionary dict, FILE * fp)
+static const char * get_a_word(Dictionary dict, FILE * fp)
 {
-	char word[MAX_WORD+1];
+	char word[MAX_WORD+4]; /* allow for 4-byte wide chars */
 	char * s;
-	int c, j;
+	wint_t c;
+	int j;
 	do {
-		c = getc(fp);
-	} while ((c != EOF) && isspace(c));
-	if (c == EOF) return NULL;
+		c = fgetwc(fp);
+	} while ((c != WEOF) && iswspace(c));
+	if (c == WEOF) return NULL;
 
-	for (j=0; (j <= MAX_WORD-1) && (!isspace(c)) && (c != EOF); j++) {
-		word[j] = c;
-		c = getc(fp);
+	for (j=0; (j <= MAX_WORD-1) && (!iswspace(c)) && (c != WEOF);)
+	{
+		j += wctomb(&word[j], c);
+		c = fgetwc(fp);
 	}
 
-	if (j == MAX_WORD) {
+	if (j >= MAX_WORD) {
 		error("The dictionary contains a word that is too long.");
 	}
 	word[j] = '\0';
@@ -53,7 +58,7 @@ Dict_node * read_word_file(Dictionary dict, Dict_node * dn, char * filename)
 	Dict_node * dn_new;
 	Word_file * wf;
 	FILE * fp;
-	char * s;
+	const char * s;
 	char file_name_copy[MAX_PATH_NAME+1];
 
 	safe_strcpy(file_name_copy, filename+1, sizeof(file_name_copy)); /* get rid of leading '/' */
@@ -93,9 +98,9 @@ void save_files(Dictionary dict)
 	for (wf = dict->word_file_header; wf != NULL; wf = wf->next) {
 		if (wf->changed) {
 			if ((fp = fopen(wf->file, "w")) == NULL) {
-			 printf("\nCannot open %s. Gee, this shouldn't happen.\n", wf->file);
-			 printf("file not saved\n");
-			 return;
+				printf("\nCannot open %s. Gee, this shouldn't happen.\n", wf->file);
+				printf("file not saved\n");
+				return;
 			}
 			printf("   saving file \"%s\"\n", wf->file);
 			/*output_dictionary(dict_root, fp, wf);*/
